"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from paygentic_sdk import errors, models, utils
from paygentic_sdk._hooks import HookContext
from paygentic_sdk.types import BaseModel, OptionalNullable, UNSET
from paygentic_sdk.utils import get_security_from_env
from paygentic_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union, cast


class Subscriptions(BaseSDK):
    r"""A `Subscription` is a customer's commitment to purchase a `Product` following the terms of a `Plan` and its linked `Prices`."""

    def list(
        self,
        *,
        request: Union[
            models.ListSubscriptionsRequest, models.ListSubscriptionsRequestTypedDict
        ] = models.ListSubscriptionsRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListSubscriptionsResponse:
        r"""List

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.ListSubscriptionsRequest)
        request = cast(models.ListSubscriptionsRequest, request)

        req = self._build_request(
            method="GET",
            path="/v0/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listSubscriptions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListSubscriptionsResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        request: Union[
            models.ListSubscriptionsRequest, models.ListSubscriptionsRequestTypedDict
        ] = models.ListSubscriptionsRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListSubscriptionsResponse:
        r"""List

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.ListSubscriptionsRequest)
        request = cast(models.ListSubscriptionsRequest, request)

        req = self._build_request_async(
            method="GET",
            path="/v0/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listSubscriptions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListSubscriptionsResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    def create(
        self,
        *,
        name: str,
        plan_id: str,
        started_at: datetime,
        auto_charge: Optional[bool] = False,
        tax_exempt: Optional[bool] = False,
        customer: Optional[
            Union[
                models.CreateSubscriptionCustomer,
                models.CreateSubscriptionCustomerTypedDict,
            ]
        ] = None,
        customer_id: Optional[str] = None,
        ending_at: Optional[datetime] = None,
        prefund_amount: Optional[str] = None,
        minimum_account_balance: Optional[str] = None,
        redirect_urls: Optional[
            Union[models.RedirectUrls, models.RedirectUrlsTypedDict]
        ] = None,
        test_clock_id: Optional[str] = None,
        renewal_reminder_enabled: OptionalNullable[bool] = UNSET,
        renewal_reminder_days: OptionalNullable[int] = UNSET,
        session_expiry_minutes: Optional[float] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Subscription:
        r"""Create

        Creates a subscription for an existing customer or creates a new customer as part of the action.

        :param name: Subscription identifier combining customer and service details. Sample values: 'TechCorp - LLM API Access', 'Analytics Co - Data Platform Enterprise', 'StartupXYZ - Image Generation Service', 'Enterprise Inc - ML Training Platform'
        :param plan_id: Unique identifier for a plan
        :param started_at: Subscription activation timestamp in ISO 8601 format. Sample values: '2024-01-15T10:30:00Z', '2024-02-01T00:00:00Z'
        :param auto_charge: Enable automatic charging of invoices using stored payment methods. When true, invoices will be automatically paid using off-session payment. Defaults to false.
        :param tax_exempt: When true, forces tax rate to 0%. Use for customers with verified tax-exempt status.
        :param customer: Fields to create a new customer and consumer. Will use an existing consumer if one exists with the same email address. Required if `customerId` is not provided. Address with complete tax information (country, state, zipCode) is required for tax calculation when using Paygentic Tax.
        :param customer_id: Unique identifier for a customer
        :param ending_at: Subscription expiration timestamp in ISO 8601 format. Sample values: '2024-12-31T23:59:59Z', '2025-01-15T10:30:00Z'. Omit for indefinite subscriptions.
        :param prefund_amount: @deprecated Use minimumAccountBalance instead. Required initial wallet deposit amount. Sample values: '200.00' requires $200 prepaid balance for metered LLM usage, '1000.00' requires $1000 prepaid credits for data processing services, '50.00' requires $50 minimum for API call consumption
        :param minimum_account_balance: Minimum wallet balance requirement in decimal dollars (e.g., '100.00'). Can be set to '0' to disable. The system will calculate the difference between this minimum and the customer's current balance, charging only what's needed to reach the minimum. If the customer already has sufficient balance, no charge is made. Note: If the calculated charge amount is below payment processor minimums (typically $1.00), it will be automatically adjusted upward to meet the minimum requirement. Sample values: '200.00' requires minimum $200 balance for metered LLM usage, '1000.00' requires minimum $1000 balance for data processing services
        :param redirect_urls: Optional redirect URLs after payment completion or failure. If not provided, uses default platform behavior.
        :param test_clock_id: Test clock identifier for simulating time-based billing scenarios. Sample values: 'tc_abc123xyz', 'tc_789def456'. Restricted to non-production environments (local, dev, sandbox). Must belong to the same merchant organization.
        :param renewal_reminder_enabled: Override plan setting for renewal reminder emails. When set, this subscription's setting takes precedence over the plan default. Set to true to enable reminders, false to disable, or null/omit to use plan default.
        :param renewal_reminder_days: Override plan setting for number of days before renewal to send the reminder. Only used if renewalReminderEnabled is true (or inherited from plan). Set to null to use plan default.
        :param session_expiry_minutes: Number of minutes until the payment session expires. Defaults to 240 minutes (4 hours) if not provided.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateSubscriptionRequest(
            auto_charge=auto_charge,
            tax_exempt=tax_exempt,
            customer=utils.get_pydantic_model(
                customer, Optional[models.CreateSubscriptionCustomer]
            ),
            customer_id=customer_id,
            ending_at=ending_at,
            name=name,
            plan_id=plan_id,
            prefund_amount=prefund_amount,
            minimum_account_balance=minimum_account_balance,
            redirect_urls=utils.get_pydantic_model(
                redirect_urls, Optional[models.RedirectUrls]
            ),
            started_at=started_at,
            test_clock_id=test_clock_id,
            renewal_reminder_enabled=renewal_reminder_enabled,
            renewal_reminder_days=renewal_reminder_days,
            session_expiry_minutes=session_expiry_minutes,
        )

        req = self._build_request(
            method="POST",
            path="/v0/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CreateSubscriptionRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, ["200", "201"], "application/json"):
            return unmarshal_json_response(models.Subscription, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        name: str,
        plan_id: str,
        started_at: datetime,
        auto_charge: Optional[bool] = False,
        tax_exempt: Optional[bool] = False,
        customer: Optional[
            Union[
                models.CreateSubscriptionCustomer,
                models.CreateSubscriptionCustomerTypedDict,
            ]
        ] = None,
        customer_id: Optional[str] = None,
        ending_at: Optional[datetime] = None,
        prefund_amount: Optional[str] = None,
        minimum_account_balance: Optional[str] = None,
        redirect_urls: Optional[
            Union[models.RedirectUrls, models.RedirectUrlsTypedDict]
        ] = None,
        test_clock_id: Optional[str] = None,
        renewal_reminder_enabled: OptionalNullable[bool] = UNSET,
        renewal_reminder_days: OptionalNullable[int] = UNSET,
        session_expiry_minutes: Optional[float] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Subscription:
        r"""Create

        Creates a subscription for an existing customer or creates a new customer as part of the action.

        :param name: Subscription identifier combining customer and service details. Sample values: 'TechCorp - LLM API Access', 'Analytics Co - Data Platform Enterprise', 'StartupXYZ - Image Generation Service', 'Enterprise Inc - ML Training Platform'
        :param plan_id: Unique identifier for a plan
        :param started_at: Subscription activation timestamp in ISO 8601 format. Sample values: '2024-01-15T10:30:00Z', '2024-02-01T00:00:00Z'
        :param auto_charge: Enable automatic charging of invoices using stored payment methods. When true, invoices will be automatically paid using off-session payment. Defaults to false.
        :param tax_exempt: When true, forces tax rate to 0%. Use for customers with verified tax-exempt status.
        :param customer: Fields to create a new customer and consumer. Will use an existing consumer if one exists with the same email address. Required if `customerId` is not provided. Address with complete tax information (country, state, zipCode) is required for tax calculation when using Paygentic Tax.
        :param customer_id: Unique identifier for a customer
        :param ending_at: Subscription expiration timestamp in ISO 8601 format. Sample values: '2024-12-31T23:59:59Z', '2025-01-15T10:30:00Z'. Omit for indefinite subscriptions.
        :param prefund_amount: @deprecated Use minimumAccountBalance instead. Required initial wallet deposit amount. Sample values: '200.00' requires $200 prepaid balance for metered LLM usage, '1000.00' requires $1000 prepaid credits for data processing services, '50.00' requires $50 minimum for API call consumption
        :param minimum_account_balance: Minimum wallet balance requirement in decimal dollars (e.g., '100.00'). Can be set to '0' to disable. The system will calculate the difference between this minimum and the customer's current balance, charging only what's needed to reach the minimum. If the customer already has sufficient balance, no charge is made. Note: If the calculated charge amount is below payment processor minimums (typically $1.00), it will be automatically adjusted upward to meet the minimum requirement. Sample values: '200.00' requires minimum $200 balance for metered LLM usage, '1000.00' requires minimum $1000 balance for data processing services
        :param redirect_urls: Optional redirect URLs after payment completion or failure. If not provided, uses default platform behavior.
        :param test_clock_id: Test clock identifier for simulating time-based billing scenarios. Sample values: 'tc_abc123xyz', 'tc_789def456'. Restricted to non-production environments (local, dev, sandbox). Must belong to the same merchant organization.
        :param renewal_reminder_enabled: Override plan setting for renewal reminder emails. When set, this subscription's setting takes precedence over the plan default. Set to true to enable reminders, false to disable, or null/omit to use plan default.
        :param renewal_reminder_days: Override plan setting for number of days before renewal to send the reminder. Only used if renewalReminderEnabled is true (or inherited from plan). Set to null to use plan default.
        :param session_expiry_minutes: Number of minutes until the payment session expires. Defaults to 240 minutes (4 hours) if not provided.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateSubscriptionRequest(
            auto_charge=auto_charge,
            tax_exempt=tax_exempt,
            customer=utils.get_pydantic_model(
                customer, Optional[models.CreateSubscriptionCustomer]
            ),
            customer_id=customer_id,
            ending_at=ending_at,
            name=name,
            plan_id=plan_id,
            prefund_amount=prefund_amount,
            minimum_account_balance=minimum_account_balance,
            redirect_urls=utils.get_pydantic_model(
                redirect_urls, Optional[models.RedirectUrls]
            ),
            started_at=started_at,
            test_clock_id=test_clock_id,
            renewal_reminder_enabled=renewal_reminder_enabled,
            renewal_reminder_days=renewal_reminder_days,
            session_expiry_minutes=session_expiry_minutes,
        )

        req = self._build_request_async(
            method="POST",
            path="/v0/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CreateSubscriptionRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, ["200", "201"], "application/json"):
            return unmarshal_json_response(models.Subscription, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    def get(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Subscription:
        r"""Get

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetSubscriptionRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/v0/subscriptions/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Subscription, http_res)
        if utils.match_response(http_res, ["401", "403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Subscription:
        r"""Get

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetSubscriptionRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v0/subscriptions/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Subscription, http_res)
        if utils.match_response(http_res, ["401", "403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    def generate_portal_link(
        self,
        *,
        id: str,
        expires_in: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscriptionPortal:
        r"""Generate Portal Link

        Generates a secure, time-limited URL that allows customers to access their subscription data without authentication.

        :param id: The subscription ID
        :param expires_in: Token lifetime as an ISO 8601 duration. Defaults to PT10M (10 minutes). Maximum is P7D (7 days). Examples: PT10M, PT1H, P1D, P7D.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GeneratePortalLinkRequest(
            id=id,
            request_body=models.GeneratePortalLinkRequestBody(
                expires_in=expires_in,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v0/subscriptions/{id}/portal",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body if request is not None else None,
                False,
                True,
                "json",
                Optional[models.GeneratePortalLinkRequestBody],
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="generatePortalLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SubscriptionPortal, http_res)
        if utils.match_response(
            http_res, ["401", "403", "404", "429"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def generate_portal_link_async(
        self,
        *,
        id: str,
        expires_in: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscriptionPortal:
        r"""Generate Portal Link

        Generates a secure, time-limited URL that allows customers to access their subscription data without authentication.

        :param id: The subscription ID
        :param expires_in: Token lifetime as an ISO 8601 duration. Defaults to PT10M (10 minutes). Maximum is P7D (7 days). Examples: PT10M, PT1H, P1D, P7D.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GeneratePortalLinkRequest(
            id=id,
            request_body=models.GeneratePortalLinkRequestBody(
                expires_in=expires_in,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v0/subscriptions/{id}/portal",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body if request is not None else None,
                False,
                True,
                "json",
                Optional[models.GeneratePortalLinkRequestBody],
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="generatePortalLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SubscriptionPortal, http_res)
        if utils.match_response(
            http_res, ["401", "403", "404", "429"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    def terminate(
        self,
        *,
        id: str,
        reason: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Subscription:
        r"""Terminate

        Terminates a subscription with a required reason. This endpoint is for merchant-initiated termination only.

        :param id: The subscription ID
        :param reason: Cancellation explanation text. Sample values: 'Customer requested cancellation', 'Payment failure', 'Service migration', 'Contract expiration'
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TerminateSubscriptionRequest(
            id=id,
            request_body=models.TerminateSubscriptionRequestBody(
                reason=reason,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v0/subscriptions/{id}/termination",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.TerminateSubscriptionRequestBody,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="terminateSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "409", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Subscription, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(
            http_res, ["401", "403", "404", "409"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def terminate_async(
        self,
        *,
        id: str,
        reason: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Subscription:
        r"""Terminate

        Terminates a subscription with a required reason. This endpoint is for merchant-initiated termination only.

        :param id: The subscription ID
        :param reason: Cancellation explanation text. Sample values: 'Customer requested cancellation', 'Payment failure', 'Service migration', 'Contract expiration'
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TerminateSubscriptionRequest(
            id=id,
            request_body=models.TerminateSubscriptionRequestBody(
                reason=reason,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v0/subscriptions/{id}/termination",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.TerminateSubscriptionRequestBody,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="terminateSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "409", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Subscription, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(
            http_res, ["401", "403", "404", "409"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel
import pydantic
from typing import Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class PriceProperties4TypedDict(TypedDict):
    r"""Percentage pricing model"""

    max_charge: str
    r"""Ceiling amount limiting total charge regardless of percentage calculation. Sample values: '500.00' caps fees at $500 maximum, '2000.00' limits charges to $2000 per period"""
    min_charge: str
    r"""Floor amount charged regardless of percentage calculation. Sample values: '5.00' ensures minimum $5 fee, '25.00' guarantees at least $25 per billing event"""
    percentage: str
    r"""Rate expressed as decimal between 0 and 10, supporting up to 4 decimal precision. Sample values: '0.03' represents 3%, '0.075' represents 7.5%, '1.10' represents 110%"""


class PriceProperties4(BaseModel):
    r"""Percentage pricing model"""

    max_charge: Annotated[str, pydantic.Field(alias="maxCharge")]
    r"""Ceiling amount limiting total charge regardless of percentage calculation. Sample values: '500.00' caps fees at $500 maximum, '2000.00' limits charges to $2000 per period"""

    min_charge: Annotated[str, pydantic.Field(alias="minCharge")]
    r"""Floor amount charged regardless of percentage calculation. Sample values: '5.00' ensures minimum $5 fee, '25.00' guarantees at least $25 per billing event"""

    percentage: str
    r"""Rate expressed as decimal between 0 and 10, supporting up to 4 decimal precision. Sample values: '0.03' represents 3%, '0.075' represents 7.5%, '1.10' represents 110%"""


Function = Literal["linear",]
r"""Function used for volume pricing calculations"""


class ParametersTypedDict(TypedDict):
    function: Function
    r"""Function used for volume pricing calculations"""
    gradient: str
    r"""Gradient for volume pricing calculations"""
    max: str
    r"""Maximum value for volume pricing"""
    min: str
    r"""Minimum value for volume pricing"""


class Parameters(BaseModel):
    function: Function
    r"""Function used for volume pricing calculations"""

    gradient: str
    r"""Gradient for volume pricing calculations"""

    max: str
    r"""Maximum value for volume pricing"""

    min: str
    r"""Minimum value for volume pricing"""


class PriceProperties3TypedDict(TypedDict):
    r"""Volume pricing model"""

    default: str
    r"""Default value for volume pricing"""
    parameters: ParametersTypedDict


class PriceProperties3(BaseModel):
    r"""Volume pricing model"""

    default: str
    r"""Default value for volume pricing"""

    parameters: Parameters


class PriceProperties2TypedDict(TypedDict):
    r"""Dynamic pricing model"""

    max_price: str
    r"""Upper limit of the price range for dynamic pricing. Sample values: '0.10' means $0.10 per token for large volumes at peak times"""
    min_price: str
    r"""Lower limit of the price range for dynamic pricing. Sample values: '0.01' means $0.01 per token for low volumes during off-peak hours"""


class PriceProperties2(BaseModel):
    r"""Dynamic pricing model"""

    max_price: Annotated[str, pydantic.Field(alias="maxPrice")]
    r"""Upper limit of the price range for dynamic pricing. Sample values: '0.10' means $0.10 per token for large volumes at peak times"""

    min_price: Annotated[str, pydantic.Field(alias="minPrice")]
    r"""Lower limit of the price range for dynamic pricing. Sample values: '0.01' means $0.01 per token for low volumes during off-peak hours"""


class PriceProperties1TypedDict(TypedDict):
    r"""Standard pricing model"""

    unit_price: str
    r"""Per-unit cost in decimal format for fixed-rate pricing. Sample values: '0.00002' represents $0.00002 per token, '0.15' represents $0.15 per gigabyte stored, '0.05' represents $0.05 per API call"""


class PriceProperties1(BaseModel):
    r"""Standard pricing model"""

    unit_price: Annotated[str, pydantic.Field(alias="unitPrice")]
    r"""Per-unit cost in decimal format for fixed-rate pricing. Sample values: '0.00002' represents $0.00002 per token, '0.15' represents $0.15 per gigabyte stored, '0.05' represents $0.05 per API call"""


PricePropertiesUnionTypedDict = TypeAliasType(
    "PricePropertiesUnionTypedDict",
    Union[
        PriceProperties1TypedDict,
        PriceProperties2TypedDict,
        PriceProperties3TypedDict,
        PriceProperties4TypedDict,
    ],
)


PricePropertiesUnion = TypeAliasType(
    "PricePropertiesUnion",
    Union[PriceProperties1, PriceProperties2, PriceProperties3, PriceProperties4],
)


try:
    PriceProperties4.model_rebuild()
except NameError:
    pass
try:
    PriceProperties2.model_rebuild()
except NameError:
    pass
try:
    PriceProperties1.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from paygentic_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


PurchaseGrantResponseObject = Literal["grant_purchase",]


class PaymentSessionTypedDict(TypedDict):
    url: str
    r"""The Stripe checkout URL for the customer to complete payment."""
    expires_at: NotRequired[Nullable[datetime]]
    r"""When the payment session expires."""
    amount: NotRequired[float]
    r"""The payment amount in the currency's minor unit (e.g., cents)."""


class PaymentSession(BaseModel):
    url: str
    r"""The Stripe checkout URL for the customer to complete payment."""

    expires_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="expiresAt")
    ] = UNSET
    r"""When the payment session expires."""

    amount: Optional[float] = None
    r"""The payment amount in the currency's minor unit (e.g., cents)."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["expiresAt", "amount"])
        nullable_fields = set(["expiresAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PurchaseGrantResponseTypedDict(TypedDict):
    invoice_id: str
    r"""The invoice ID for this grant purchase."""
    entitlement_id: str
    r"""The entitlement that will receive the grant."""
    grant_amount: int
    r"""The number of credits that will be granted on payment."""
    price: str
    r"""The price in decimal format (e.g., '5.00')."""
    currency: str
    r"""The three-letter ISO 4217 currency code (e.g., 'usd')."""
    payment_sessions: List[PaymentSessionTypedDict]
    r"""Payment session(s) for the customer to complete the purchase."""
    object: NotRequired[PurchaseGrantResponseObject]


class PurchaseGrantResponse(BaseModel):
    invoice_id: Annotated[str, pydantic.Field(alias="invoiceId")]
    r"""The invoice ID for this grant purchase."""

    entitlement_id: Annotated[str, pydantic.Field(alias="entitlementId")]
    r"""The entitlement that will receive the grant."""

    grant_amount: Annotated[int, pydantic.Field(alias="grantAmount")]
    r"""The number of credits that will be granted on payment."""

    price: str
    r"""The price in decimal format (e.g., '5.00')."""

    currency: str
    r"""The three-letter ISO 4217 currency code (e.g., 'usd')."""

    payment_sessions: Annotated[
        List[PaymentSession], pydantic.Field(alias="paymentSessions")
    ]
    r"""Payment session(s) for the customer to complete the purchase."""

    object: Optional[PurchaseGrantResponseObject] = "grant_purchase"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["object"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PaymentSession.model_rebuild()
except NameError:
    pass
try:
    PurchaseGrantResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel
from paygentic_sdk.utils import FieldMetadata, PathParamMetadata
import pydantic
from typing_extensions import Annotated, TypedDict


class GetSourceRuleRequestTypedDict(TypedDict):
    rule_id: str


class GetSourceRuleRequest(BaseModel):
    rule_id: Annotated[
        str,
        pydantic.Field(alias="ruleId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class CreateFeeRequestTypedDict(TypedDict):
    name: str
    r"""Human-readable label identifying the fee. Sample values: 'Setup Fee', 'Monthly Subscription', 'Compliance Update', 'Annual License'"""
    description: str
    r"""Explanatory text describing what the fee represents. Sample values: 'One-time setup fee for new customers', 'Monthly base subscription charge', 'Yearly compliance and security update fee'"""
    merchant_id: str
    r"""Unique identifier for an organization"""
    product_id: str
    r"""Unique identifier for a product"""


class CreateFeeRequest(BaseModel):
    name: str
    r"""Human-readable label identifying the fee. Sample values: 'Setup Fee', 'Monthly Subscription', 'Compliance Update', 'Annual License'"""

    description: str
    r"""Explanatory text describing what the fee represents. Sample values: 'One-time setup fee for new customers', 'Monthly base subscription charge', 'Yearly compliance and security update fee'"""

    merchant_id: Annotated[str, pydantic.Field(alias="merchantId")]
    r"""Unique identifier for an organization"""

    product_id: Annotated[str, pydantic.Field(alias="productId")]
    r"""Unique identifier for a product"""


try:
    CreateFeeRequest.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import Any, TYPE_CHECKING

from paygentic_sdk.utils.dynamic_imports import lazy_getattr, lazy_dir

if TYPE_CHECKING:
    from .activeentitlement import (
        ActiveEntitlement,
        ActiveEntitlementObject,
        ActiveEntitlementTypedDict,
    )
    from .address import Address, AddressTypedDict
    from .advancetestclockop import (
        AdvanceTestClockRequest,
        AdvanceTestClockRequestBody,
        AdvanceTestClockRequestBodyTypedDict,
        AdvanceTestClockRequestTypedDict,
        RequestBody1,
        RequestBody1TypedDict,
        RequestBody2,
        RequestBody2TypedDict,
    )
    from .approvesourceeventop import (
        ApproveSourceEventRequest,
        ApproveSourceEventRequestTypedDict,
    )
    from .batchcreateusageeventsop import (
        BatchCreateUsageEventsRequest,
        BatchCreateUsageEventsRequestTypedDict,
    )
    from .batchusageeventresponse import (
        BatchUsageEventResponse,
        BatchUsageEventResponseObject,
        BatchUsageEventResponseTypedDict,
        Failed,
        FailedTypedDict,
        Summary,
        SummaryTypedDict,
    )
    from .billablemetric import (
        Aggregation,
        BillableMetric,
        BillableMetricObject,
        BillableMetricTypedDict,
    )
    from .booleanentitlementdetail import (
        BooleanEntitlementDetail,
        BooleanEntitlementDetailObject,
        BooleanEntitlementDetailStatus,
        BooleanEntitlementDetailTypedDict,
    )
    from .bulkapprovesourceeventsop import (
        BulkApproveSourceEventsDetails,
        BulkApproveSourceEventsDetailsTypedDict,
        BulkApproveSourceEventsFailed,
        BulkApproveSourceEventsFailedTypedDict,
        BulkApproveSourceEventsRequest,
        BulkApproveSourceEventsRequestBody,
        BulkApproveSourceEventsRequestBodyTypedDict,
        BulkApproveSourceEventsRequestTypedDict,
        BulkApproveSourceEventsResponse,
        BulkApproveSourceEventsResponseTypedDict,
    )
    from .bulkrejectsourceeventsop import (
        BulkRejectSourceEventsRequest,
        BulkRejectSourceEventsRequestBody,
        BulkRejectSourceEventsRequestBodyTypedDict,
        BulkRejectSourceEventsRequestTypedDict,
        BulkRejectSourceEventsResponse,
        BulkRejectSourceEventsResponseTypedDict,
    )
    from .createbillablemetricop import (
        CreateBillableMetricAggregation,
        CreateBillableMetricRequest,
        CreateBillableMetricRequestTypedDict,
    )
    from .createcustomerop import (
        Consumer,
        ConsumerTypedDict,
        CreateCustomerRequest,
        CreateCustomerRequestTypedDict,
        CreateCustomerResponse,
        CreateCustomerResponseBody1,
        CreateCustomerResponseBody1TypedDict,
        CreateCustomerResponseBody2,
        CreateCustomerResponseBody2TypedDict,
        CreateCustomerResponseTypedDict,
    )
    from .createdisputerequest import (
        CreateDisputeRequest,
        CreateDisputeRequestTypedDict,
    )
    from .createentitlementgrantop import (
        CreateEntitlementGrantRequest,
        CreateEntitlementGrantRequestTypedDict,
    )
    from .createentitlementrequest import (
        CreateEntitlementRequest,
        CreateEntitlementRequestTypedDict,
        EntitlementDatum,
        EntitlementDatumTypedDict,
    )
    from .createfeatureop import (
        CreateFeatureRequest,
        CreateFeatureRequestTypedDict,
        CreateFeatureType,
    )
    from .createfeeop import CreateFeeRequest, CreateFeeRequestTypedDict
    from .creategrantrequest import CreateGrantRequest, CreateGrantRequestTypedDict
    from .createlineitemop import CreateLineItemRequest, CreateLineItemRequestTypedDict
    from .createmanuallineitemrequest import (
        CreateManualLineItemRequest,
        CreateManualLineItemRequestTypedDict,
    )
    from .createpaymentop import (
        CreatePaymentLineItem,
        CreatePaymentLineItemTypedDict,
        CreatePaymentRequest,
        CreatePaymentRequestTypedDict,
        Currency,
    )
    from .createplanop import (
        BillingVersion,
        CreatePlanRequest,
        CreatePlanRequestTypedDict,
        CreatePlanTaxBehavior,
    )
    from .createpriceop import (
        CreatePriceModel,
        CreatePricePaymentTerm,
        CreatePriceRequest,
        CreatePriceRequestTypedDict,
    )
    from .createproductop import (
        CreateProductRequest,
        CreateProductRequestTypedDict,
        Metadata,
        MetadataTypedDict,
    )
    from .createrulerequest import CreateRuleRequest, CreateRuleRequestTypedDict
    from .createsourceop import (
        CreateSourceProcessingMode,
        CreateSourceRequest,
        CreateSourceRequestTypedDict,
        CreateSourceType,
    )
    from .createsubscriptionop import (
        CreateSubscriptionAddress,
        CreateSubscriptionAddressTypedDict,
        CreateSubscriptionCustomer,
        CreateSubscriptionCustomerTypedDict,
        CreateSubscriptionRequest,
        CreateSubscriptionRequestTypedDict,
        RedirectUrls,
        RedirectUrlsTypedDict,
    )
    from .createtestclockop import (
        CreateTestClockRequest,
        CreateTestClockRequestTypedDict,
    )
    from .createusageeventrequest import (
        CreateUsageEventRequest,
        CreateUsageEventRequestProperty,
        CreateUsageEventRequestPropertyTypedDict,
        CreateUsageEventRequestTypedDict,
    )
    from .customer import (
        Customer,
        CustomerObject,
        CustomerOrganization,
        CustomerOrganizationTypedDict,
        CustomerTypedDict,
    )
    from .deletecustomerop import (
        DeleteCustomerBlocker,
        DeleteCustomerBlockerTypedDict,
        DeleteCustomerDetails,
        DeleteCustomerDetailsTypedDict,
        DeleteCustomerError,
        DeleteCustomerRequest,
        DeleteCustomerRequestTypedDict,
        DeleteCustomerType,
        Item,
        ItemTypedDict,
    )
    from .deletefeatureop import DeleteFeatureRequest, DeleteFeatureRequestTypedDict
    from .deletefeeop import (
        DeleteFeeBlocker,
        DeleteFeeBlockerTypedDict,
        DeleteFeeDetails,
        DeleteFeeDetailsTypedDict,
        DeleteFeeRequest,
        DeleteFeeRequestTypedDict,
    )
    from .deletepriceop import DeletePriceRequest, DeletePriceRequestTypedDict
    from .deletesourceruleop import (
        DeleteSourceRuleRequest,
        DeleteSourceRuleRequestTypedDict,
    )
    from .deletetestclockop import (
        DeleteTestClockRequest,
        DeleteTestClockRequestTypedDict,
    )
    from .dispute import Dispute, DisputeObject, DisputeStatus, DisputeTypedDict
    from .disputewithcustomer import (
        DisputeWithCustomer,
        DisputeWithCustomerCustomer,
        DisputeWithCustomerCustomerTypedDict,
        DisputeWithCustomerObject,
        DisputeWithCustomerStatus,
        DisputeWithCustomerTypedDict,
    )
    from .entitlement import (
        Entitlement,
        EntitlementBillableMetric,
        EntitlementBillableMetricTypedDict,
        EntitlementObject,
        EntitlementTypedDict,
    )
    from .entitlementaccessresult import (
        EntitlementAccessResult,
        EntitlementAccessResultStatus,
        EntitlementAccessResultTypedDict,
        FeatureTypeEnum,
    )
    from .entitlementdetail import (
        EntitlementDetail,
        EntitlementDetailTypedDict,
        UnknownEntitlementDetail,
    )
    from .entitlementtemplate import (
        EntitlementTemplate,
        EntitlementTemplateBoolean,
        EntitlementTemplateBooleanTypedDict,
        EntitlementTemplateMetered,
        EntitlementTemplateMeteredTypedDict,
        EntitlementTemplateStatic,
        EntitlementTemplateStaticTypedDict,
        EntitlementTemplateTypedDict,
        UsagePeriod,
        UsagePeriodTypedDict,
    )
    from .eventresponse import (
        EventResponse,
        EventResponseObject,
        EventResponseTypedDict,
    )
    from .feature import Feature, FeatureObject, FeatureType, FeatureTypedDict
    from .fee import Fee, FeeObject, FeeTypedDict
    from .feeprice import (
        FeePrice,
        FeePriceModel,
        FeePricePaymentTerm,
        FeePriceTypedDict,
        Properties,
        PropertiesTypedDict,
    )
    from .generateportallinkop import (
        GeneratePortalLinkRequest,
        GeneratePortalLinkRequestBody,
        GeneratePortalLinkRequestBodyTypedDict,
        GeneratePortalLinkRequestTypedDict,
    )
    from .getactiveentitlementsop import (
        GetActiveEntitlementsObject,
        GetActiveEntitlementsPagination,
        GetActiveEntitlementsPaginationTypedDict,
        GetActiveEntitlementsRequest,
        GetActiveEntitlementsRequestTypedDict,
        GetActiveEntitlementsResponse,
        GetActiveEntitlementsResponseTypedDict,
    )
    from .getbillablemetricmeterop import (
        GetBillableMetricMeterRequest,
        GetBillableMetricMeterRequestTypedDict,
        WindowSize,
    )
    from .getbillablemetricop import (
        GetBillableMetricRequest,
        GetBillableMetricRequestTypedDict,
    )
    from .getcustomerop import GetCustomerRequest, GetCustomerRequestTypedDict
    from .getentitlementgrantop import (
        GetEntitlementGrantRequest,
        GetEntitlementGrantRequestTypedDict,
    )
    from .getentitlementop import GetEntitlementRequest, GetEntitlementRequestTypedDict
    from .getfeatureop import GetFeatureRequest, GetFeatureRequestTypedDict
    from .getfeeop import GetFeeRequest, GetFeeRequestTypedDict
    from .getfeepriceop import GetFeePriceRequest, GetFeePriceRequestTypedDict
    from .getinvoicelineitemsop import (
        GetInvoiceLineItemsRequest,
        GetInvoiceLineItemsRequestTypedDict,
    )
    from .getinvoiceop import GetInvoiceRequest, GetInvoiceRequestTypedDict
    from .getpaymentop import GetPaymentRequest, GetPaymentRequestTypedDict
    from .getplanop import GetPlanRequest, GetPlanRequestTypedDict
    from .getpriceop import GetPriceRequest, GetPriceRequestTypedDict
    from .getproductop import GetProductRequest, GetProductRequestTypedDict
    from .getrevenueop import (
        BucketWidth,
        GetRevenueRequest,
        GetRevenueRequestTypedDict,
        GroupBy,
    )
    from .getsourceop import GetSourceRequest, GetSourceRequestTypedDict
    from .getsourceruleop import GetSourceRuleRequest, GetSourceRuleRequestTypedDict
    from .getsubscriptionop import (
        GetSubscriptionRequest,
        GetSubscriptionRequestTypedDict,
    )
    from .gettestclockop import GetTestClockRequest, GetTestClockRequestTypedDict
    from .getusageeventop import GetUsageEventRequest, GetUsageEventRequestTypedDict
    from .getuserop import GetUserRequest, GetUserRequestTypedDict
    from .grant import Grant, GrantObject, GrantTypedDict
    from .groupinvoicesummary import GroupInvoiceSummary, GroupInvoiceSummaryTypedDict
    from .grouptrendentry import GroupTrendEntry, GroupTrendEntryTypedDict
    from .ingesteventop import IngestEventRequest, IngestEventRequestTypedDict
    from .invoice import (
        Invoice,
        InvoiceObject,
        InvoiceStatus,
        InvoiceTypedDict,
        LineItems,
        LineItemsTypedDict,
        Tax,
        TaxTypedDict,
    )
    from .invoicecategorysummary import (
        InvoiceCategorySummary,
        InvoiceCategorySummaryTypedDict,
    )
    from .invoicelineitem import (
        EventType,
        InvoiceLineItem,
        InvoiceLineItemLineItemType,
        InvoiceLineItemTypedDict,
    )
    from .invoicelineitemsresponse import (
        InvoiceLineItemsResponse,
        InvoiceLineItemsResponseTypedDict,
    )
    from .invoicesummary import InvoiceSummary, InvoiceSummaryTypedDict
    from .issueentitlementrequest import (
        IssueEntitlementRequest,
        IssueEntitlementRequestTypedDict,
    )
    from .lineitem import (
        LineItem,
        LineItemPaymentTerm,
        LineItemStatus,
        LineItemType,
        LineItemTypedDict,
    )
    from .lineitemsresponse import (
        LineItemsResponse,
        LineItemsResponseObject,
        LineItemsResponseTypedDict,
    )
    from .lineitemssummary import LineItemsSummary, LineItemsSummaryTypedDict
    from .listavailableplansop import (
        ListAvailablePlansPagination,
        ListAvailablePlansPaginationTypedDict,
        ListAvailablePlansRequest,
        ListAvailablePlansRequestTypedDict,
        ListAvailablePlansResponse,
        ListAvailablePlansResponseTypedDict,
    )
    from .listbillablemetricsop import (
        ListBillableMetricsObject,
        ListBillableMetricsPagination,
        ListBillableMetricsPaginationTypedDict,
        ListBillableMetricsRequest,
        ListBillableMetricsRequestTypedDict,
        ListBillableMetricsResponse,
        ListBillableMetricsResponseTypedDict,
    )
    from .listcustomersop import (
        ListCustomersObject,
        ListCustomersRequest,
        ListCustomersRequestTypedDict,
        ListCustomersResponse,
        ListCustomersResponseTypedDict,
    )
    from .listdisputesop import (
        ListDisputesObject,
        ListDisputesPagination,
        ListDisputesPaginationTypedDict,
        ListDisputesRequest,
        ListDisputesRequestTypedDict,
        ListDisputesResponse,
        ListDisputesResponseTypedDict,
        ListDisputesStatus,
    )
    from .listentitlementgrantsop import (
        ListEntitlementGrantsObject,
        ListEntitlementGrantsRequest,
        ListEntitlementGrantsRequestTypedDict,
        ListEntitlementGrantsResponse,
        ListEntitlementGrantsResponseTypedDict,
    )
    from .listentitlementsop import (
        ListEntitlementsObject,
        ListEntitlementsRequest,
        ListEntitlementsRequestTypedDict,
        ListEntitlementsResponse,
        ListEntitlementsResponseTypedDict,
    )
    from .listfeaturesop import (
        ListFeaturesObject,
        ListFeaturesPagination,
        ListFeaturesPaginationTypedDict,
        ListFeaturesRequest,
        ListFeaturesRequestTypedDict,
        ListFeaturesResponse,
        ListFeaturesResponseTypedDict,
    )
    from .listfeesop import (
        ListFeesObject,
        ListFeesPagination,
        ListFeesPaginationTypedDict,
        ListFeesRequest,
        ListFeesRequestTypedDict,
        ListFeesResponse,
        ListFeesResponseTypedDict,
    )
    from .listinvoicesop import (
        ListInvoicesObject,
        ListInvoicesRequest,
        ListInvoicesRequestTypedDict,
        ListInvoicesResponse,
        ListInvoicesResponseTypedDict,
        ListInvoicesStatus,
        NextActionAt,
    )
    from .listlineitemsop import (
        ListLineItemsRequest,
        ListLineItemsRequestTypedDict,
        ListLineItemsStatus,
    )
    from .listpaymentsop import (
        ListPaymentsObject,
        ListPaymentsRequest,
        ListPaymentsRequestTypedDict,
        ListPaymentsResponse,
        ListPaymentsResponseTypedDict,
        ListPaymentsStatus,
    )
    from .listplansop import (
        ListPlansPagination,
        ListPlansPaginationTypedDict,
        ListPlansRequest,
        ListPlansRequestTypedDict,
        ListPlansResponse,
        ListPlansResponseTypedDict,
    )
    from .listpricesop import (
        ListPricesPagination,
        ListPricesPaginationTypedDict,
        ListPricesRequest,
        ListPricesRequestTypedDict,
        ListPricesResponse,
        ListPricesResponseTypedDict,
    )
    from .listproductsop import (
        ListProductsRequest,
        ListProductsRequestTypedDict,
        ListProductsResponse,
        ListProductsResponseTypedDict,
    )
    from .listsourceeventsop import (
        ListSourceEventsRequest,
        ListSourceEventsRequestTypedDict,
        ListSourceEventsResponse,
        ListSourceEventsResponseTypedDict,
        ListSourceEventsStatus,
    )
    from .listsourcerulesop import (
        ListSourceRulesRequest,
        ListSourceRulesRequestTypedDict,
        ListSourceRulesResponse,
        ListSourceRulesResponseTypedDict,
    )
    from .listsourcesop import (
        ListSourcesRequest,
        ListSourcesRequestTypedDict,
        ListSourcesResponse,
        ListSourcesResponseTypedDict,
    )
    from .listsubscriptionsop import (
        ListSubscriptionsRequest,
        ListSubscriptionsRequestTypedDict,
        ListSubscriptionsResponse,
        ListSubscriptionsResponseTypedDict,
        ListSubscriptionsStatus,
    )
    from .listtestclocksop import (
        ListTestClocksPagination,
        ListTestClocksPaginationTypedDict,
        ListTestClocksRequest,
        ListTestClocksRequestTypedDict,
        ListTestClocksResponse,
        ListTestClocksResponseTypedDict,
    )
    from .listusageeventsop import (
        ListUsageEventsObject,
        ListUsageEventsRequest,
        ListUsageEventsRequestTypedDict,
        ListUsageEventsResponse,
        ListUsageEventsResponseTypedDict,
    )
    from .meteredentitlementdetail import (
        MeteredEntitlementDetail,
        MeteredEntitlementDetailObject,
        MeteredEntitlementDetailStatus,
        MeteredEntitlementDetailTypedDict,
    )
    from .offsetpagination import OffsetPagination, OffsetPaginationTypedDict
    from .payment import (
        Payment,
        PaymentLineItem,
        PaymentLineItemTypedDict,
        PaymentObject,
        PaymentStatus,
        PaymentTypedDict,
    )
    from .paymentsummary import PaymentSummary, PaymentSummaryTypedDict
    from .plan import (
        Plan,
        PlanObject,
        PlanPaymentTerm,
        PlanPaymentTermTypedDict,
        PlanTypedDict,
        TaxBehavior,
    )
    from .price import Price, PriceModel, PriceObject, PricePaymentTerm, PriceTypedDict
    from .pricefeature import (
        PriceFeature,
        PriceFeatureFeature,
        PriceFeatureFeatureTypedDict,
        PriceFeatureType,
        PriceFeatureTypedDict,
    )
    from .pricefeatureinput import PriceFeatureInput, PriceFeatureInputTypedDict
    from .priceproperties_union import (
        Function,
        Parameters,
        ParametersTypedDict,
        PriceProperties1,
        PriceProperties1TypedDict,
        PriceProperties2,
        PriceProperties2TypedDict,
        PriceProperties3,
        PriceProperties3TypedDict,
        PriceProperties4,
        PriceProperties4TypedDict,
        PricePropertiesUnion,
        PricePropertiesUnionTypedDict,
    )
    from .product import Product, ProductObject, ProductTypedDict
    from .purchaseentitlementgrantop import (
        PurchaseEntitlementGrantRequest,
        PurchaseEntitlementGrantRequestTypedDict,
    )
    from .purchasegrantrequest import (
        PurchaseGrantRequest,
        PurchaseGrantRequestTypedDict,
    )
    from .purchasegrantresponse import (
        PaymentSession,
        PaymentSessionTypedDict,
        PurchaseGrantResponse,
        PurchaseGrantResponseObject,
        PurchaseGrantResponseTypedDict,
    )
    from .refundusageeventop import (
        RefundUsageEventRequest,
        RefundUsageEventRequestBody,
        RefundUsageEventRequestBodyTypedDict,
        RefundUsageEventRequestTypedDict,
    )
    from .rejectsourceeventop import (
        RejectSourceEventRequest,
        RejectSourceEventRequestTypedDict,
    )
    from .revenuesummaryresponse import (
        RevenueSummaryResponse,
        RevenueSummaryResponseTypedDict,
    )
    from .revenuetrendbucket import RevenueTrendBucket, RevenueTrendBucketTypedDict
    from .rulecondition import (
        From,
        FromTypedDict,
        Operator,
        RuleCondition,
        RuleConditionType,
        RuleConditionTypedDict,
        To,
        ToTypedDict,
        Value,
        ValueTypedDict,
        ValueUnion,
        ValueUnionTypedDict,
    )
    from .schemas_entitlement import (
        SchemasEntitlement,
        SchemasEntitlementObject,
        SchemasEntitlementStatus,
        SchemasEntitlementTypedDict,
    )
    from .schemas_price import (
        SchemasPrice,
        SchemasPriceModel,
        SchemasPriceObject,
        SchemasPricePaymentTerm,
        SchemasPriceTypedDict,
    )
    from .security import Security, SecurityTypedDict
    from .source import (
        ProcessingMode,
        Source,
        SourceObject,
        SourceType,
        SourceTypedDict,
    )
    from .sourceevent import SourceEvent, SourceEventStatus, SourceEventTypedDict
    from .sourcerule import SourceRule, SourceRuleTypedDict
    from .staticentitlementdetail import (
        StaticEntitlementDetail,
        StaticEntitlementDetailObject,
        StaticEntitlementDetailStatus,
        StaticEntitlementDetailTypedDict,
    )
    from .subscription import (
        Breakdown,
        BreakdownTypedDict,
        PaymentPaid,
        PaymentPaidTypedDict,
        PaymentPending,
        PaymentPendingTypedDict,
        PaymentUnion,
        PaymentUnionTypedDict,
        Subscription,
        SubscriptionObject,
        SubscriptionStatusEnum,
        SubscriptionTypedDict,
        UnknownPaymentUnion,
    )
    from .subscriptionportal import (
        SubscriptionPortal,
        SubscriptionPortalObject,
        SubscriptionPortalTypedDict,
    )
    from .terminatesubscriptionop import (
        TerminateSubscriptionRequest,
        TerminateSubscriptionRequestBody,
        TerminateSubscriptionRequestBodyTypedDict,
        TerminateSubscriptionRequestTypedDict,
    )
    from .testclock import TestClock, TestClockTypedDict
    from .updatebillablemetricop import (
        UpdateBillableMetricRequest,
        UpdateBillableMetricRequestBody,
        UpdateBillableMetricRequestBodyTypedDict,
        UpdateBillableMetricRequestTypedDict,
    )
    from .updatecustomerop import (
        TaxRates,
        TaxRatesTypedDict,
        UpdateCustomerRequest,
        UpdateCustomerRequestBody,
        UpdateCustomerRequestBodyTypedDict,
        UpdateCustomerRequestTypedDict,
    )
    from .updatefeatureop import (
        UpdateFeatureRequest,
        UpdateFeatureRequestBody,
        UpdateFeatureRequestBodyTypedDict,
        UpdateFeatureRequestTypedDict,
        UpdateFeatureType,
    )
    from .updatefeeop import (
        UpdateFeeRequest,
        UpdateFeeRequestBody,
        UpdateFeeRequestBodyTypedDict,
        UpdateFeeRequestTypedDict,
    )
    from .updateplanop import (
        UpdatePlanRequest,
        UpdatePlanRequestBody,
        UpdatePlanRequestBodyTypedDict,
        UpdatePlanRequestTypedDict,
        UpdatePlanTaxBehavior,
    )
    from .updatepriceop import (
        UpdatePriceModel,
        UpdatePricePaymentTerm,
        UpdatePriceRequest,
        UpdatePriceRequestBody,
        UpdatePriceRequestBodyTypedDict,
        UpdatePriceRequestTypedDict,
    )
    from .updateproductop import (
        UpdateProductRequest,
        UpdateProductRequestBody,
        UpdateProductRequestBodyTypedDict,
        UpdateProductRequestTypedDict,
    )
    from .updaterulerequest import UpdateRuleRequest, UpdateRuleRequestTypedDict
    from .updatesourceop import (
        UpdateSourceProcessingMode,
        UpdateSourceRequest,
        UpdateSourceRequestBody,
        UpdateSourceRequestBodyTypedDict,
        UpdateSourceRequestTypedDict,
    )
    from .updatesourceruleop import (
        UpdateSourceRuleRequest,
        UpdateSourceRuleRequestTypedDict,
    )
    from .updateuserop import (
        UpdateUserRequest,
        UpdateUserRequestBody,
        UpdateUserRequestBodyTypedDict,
        UpdateUserRequestTypedDict,
        UpdateUserType,
    )
    from .usageevent import (
        Billing,
        BillingTypedDict,
        Refund,
        RefundTypedDict,
        UsageEvent,
        UsageEventObject,
        UsageEventProperty,
        UsageEventPropertyTypedDict,
        UsageEventTypedDict,
    )
    from .usageresponse import (
        GroupedValue,
        GroupedValueTypedDict,
        UsageResponse,
        UsageResponseObject,
        UsageResponseTypedDict,
        WindowedValue,
        WindowedValueTypedDict,
    )
    from .user import (
        State,
        User,
        UserObject,
        UserOrganization,
        UserOrganizationTypedDict,
        UserType,
        UserTypedDict,
    )
    from .validationerror import Error, ErrorEnum, ErrorTypedDict
    from .validtaxaddress import ValidTaxAddress, ValidTaxAddressTypedDict
    from .voidentitlementgrantop import (
        VoidEntitlementGrantRequest,
        VoidEntitlementGrantRequestTypedDict,
    )

__all__ = [
    "ActiveEntitlement",
    "ActiveEntitlementObject",
    "ActiveEntitlementTypedDict",
    "Address",
    "AddressTypedDict",
    "AdvanceTestClockRequest",
    "AdvanceTestClockRequestBody",
    "AdvanceTestClockRequestBodyTypedDict",
    "AdvanceTestClockRequestTypedDict",
    "Aggregation",
    "ApproveSourceEventRequest",
    "ApproveSourceEventRequestTypedDict",
    "BatchCreateUsageEventsRequest",
    "BatchCreateUsageEventsRequestTypedDict",
    "BatchUsageEventResponse",
    "BatchUsageEventResponseObject",
    "BatchUsageEventResponseTypedDict",
    "BillableMetric",
    "BillableMetricObject",
    "BillableMetricTypedDict",
    "Billing",
    "BillingTypedDict",
    "BillingVersion",
    "BooleanEntitlementDetail",
    "BooleanEntitlementDetailObject",
    "BooleanEntitlementDetailStatus",
    "BooleanEntitlementDetailTypedDict",
    "Breakdown",
    "BreakdownTypedDict",
    "BucketWidth",
    "BulkApproveSourceEventsDetails",
    "BulkApproveSourceEventsDetailsTypedDict",
    "BulkApproveSourceEventsFailed",
    "BulkApproveSourceEventsFailedTypedDict",
    "BulkApproveSourceEventsRequest",
    "BulkApproveSourceEventsRequestBody",
    "BulkApproveSourceEventsRequestBodyTypedDict",
    "BulkApproveSourceEventsRequestTypedDict",
    "BulkApproveSourceEventsResponse",
    "BulkApproveSourceEventsResponseTypedDict",
    "BulkRejectSourceEventsRequest",
    "BulkRejectSourceEventsRequestBody",
    "BulkRejectSourceEventsRequestBodyTypedDict",
    "BulkRejectSourceEventsRequestTypedDict",
    "BulkRejectSourceEventsResponse",
    "BulkRejectSourceEventsResponseTypedDict",
    "Consumer",
    "ConsumerTypedDict",
    "CreateBillableMetricAggregation",
    "CreateBillableMetricRequest",
    "CreateBillableMetricRequestTypedDict",
    "CreateCustomerRequest",
    "CreateCustomerRequestTypedDict",
    "CreateCustomerResponse",
    "CreateCustomerResponseBody1",
    "CreateCustomerResponseBody1TypedDict",
    "CreateCustomerResponseBody2",
    "CreateCustomerResponseBody2TypedDict",
    "CreateCustomerResponseTypedDict",
    "CreateDisputeRequest",
    "CreateDisputeRequestTypedDict",
    "CreateEntitlementGrantRequest",
    "CreateEntitlementGrantRequestTypedDict",
    "CreateEntitlementRequest",
    "CreateEntitlementRequestTypedDict",
    "CreateFeatureRequest",
    "CreateFeatureRequestTypedDict",
    "CreateFeatureType",
    "CreateFeeRequest",
    "CreateFeeRequestTypedDict",
    "CreateGrantRequest",
    "CreateGrantRequestTypedDict",
    "CreateLineItemRequest",
    "CreateLineItemRequestTypedDict",
    "CreateManualLineItemRequest",
    "CreateManualLineItemRequestTypedDict",
    "CreatePaymentLineItem",
    "CreatePaymentLineItemTypedDict",
    "CreatePaymentRequest",
    "CreatePaymentRequestTypedDict",
    "CreatePlanRequest",
    "CreatePlanRequestTypedDict",
    "CreatePlanTaxBehavior",
    "CreatePriceModel",
    "CreatePricePaymentTerm",
    "CreatePriceRequest",
    "CreatePriceRequestTypedDict",
    "CreateProductRequest",
    "CreateProductRequestTypedDict",
    "CreateRuleRequest",
    "CreateRuleRequestTypedDict",
    "CreateSourceProcessingMode",
    "CreateSourceRequest",
    "CreateSourceRequestTypedDict",
    "CreateSourceType",
    "CreateSubscriptionAddress",
    "CreateSubscriptionAddressTypedDict",
    "CreateSubscriptionCustomer",
    "CreateSubscriptionCustomerTypedDict",
    "CreateSubscriptionRequest",
    "CreateSubscriptionRequestTypedDict",
    "CreateTestClockRequest",
    "CreateTestClockRequestTypedDict",
    "CreateUsageEventRequest",
    "CreateUsageEventRequestProperty",
    "CreateUsageEventRequestPropertyTypedDict",
    "CreateUsageEventRequestTypedDict",
    "Currency",
    "Customer",
    "CustomerObject",
    "CustomerOrganization",
    "CustomerOrganizationTypedDict",
    "CustomerTypedDict",
    "DeleteCustomerBlocker",
    "DeleteCustomerBlockerTypedDict",
    "DeleteCustomerDetails",
    "DeleteCustomerDetailsTypedDict",
    "DeleteCustomerError",
    "DeleteCustomerRequest",
    "DeleteCustomerRequestTypedDict",
    "DeleteCustomerType",
    "DeleteFeatureRequest",
    "DeleteFeatureRequestTypedDict",
    "DeleteFeeBlocker",
    "DeleteFeeBlockerTypedDict",
    "DeleteFeeDetails",
    "DeleteFeeDetailsTypedDict",
    "DeleteFeeRequest",
    "DeleteFeeRequestTypedDict",
    "DeletePriceRequest",
    "DeletePriceRequestTypedDict",
    "DeleteSourceRuleRequest",
    "DeleteSourceRuleRequestTypedDict",
    "DeleteTestClockRequest",
    "DeleteTestClockRequestTypedDict",
    "Dispute",
    "DisputeObject",
    "DisputeStatus",
    "DisputeTypedDict",
    "DisputeWithCustomer",
    "DisputeWithCustomerCustomer",
    "DisputeWithCustomerCustomerTypedDict",
    "DisputeWithCustomerObject",
    "DisputeWithCustomerStatus",
    "DisputeWithCustomerTypedDict",
    "Entitlement",
    "EntitlementAccessResult",
    "EntitlementAccessResultStatus",
    "EntitlementAccessResultTypedDict",
    "EntitlementBillableMetric",
    "EntitlementBillableMetricTypedDict",
    "EntitlementDatum",
    "EntitlementDatumTypedDict",
    "EntitlementDetail",
    "EntitlementDetailTypedDict",
    "EntitlementObject",
    "EntitlementTemplate",
    "EntitlementTemplateBoolean",
    "EntitlementTemplateBooleanTypedDict",
    "EntitlementTemplateMetered",
    "EntitlementTemplateMeteredTypedDict",
    "EntitlementTemplateStatic",
    "EntitlementTemplateStaticTypedDict",
    "EntitlementTemplateTypedDict",
    "EntitlementTypedDict",
    "Error",
    "ErrorEnum",
    "ErrorTypedDict",
    "EventResponse",
    "EventResponseObject",
    "EventResponseTypedDict",
    "EventType",
    "Failed",
    "FailedTypedDict",
    "Feature",
    "FeatureObject",
    "FeatureType",
    "FeatureTypeEnum",
    "FeatureTypedDict",
    "Fee",
    "FeeObject",
    "FeePrice",
    "FeePriceModel",
    "FeePricePaymentTerm",
    "FeePriceTypedDict",
    "FeeTypedDict",
    "From",
    "FromTypedDict",
    "Function",
    "GeneratePortalLinkRequest",
    "GeneratePortalLinkRequestBody",
    "GeneratePortalLinkRequestBodyTypedDict",
    "GeneratePortalLinkRequestTypedDict",
    "GetActiveEntitlementsObject",
    "GetActiveEntitlementsPagination",
    "GetActiveEntitlementsPaginationTypedDict",
    "GetActiveEntitlementsRequest",
    "GetActiveEntitlementsRequestTypedDict",
    "GetActiveEntitlementsResponse",
    "GetActiveEntitlementsResponseTypedDict",
    "GetBillableMetricMeterRequest",
    "GetBillableMetricMeterRequestTypedDict",
    "GetBillableMetricRequest",
    "GetBillableMetricRequestTypedDict",
    "GetCustomerRequest",
    "GetCustomerRequestTypedDict",
    "GetEntitlementGrantRequest",
    "GetEntitlementGrantRequestTypedDict",
    "GetEntitlementRequest",
    "GetEntitlementRequestTypedDict",
    "GetFeatureRequest",
    "GetFeatureRequestTypedDict",
    "GetFeePriceRequest",
    "GetFeePriceRequestTypedDict",
    "GetFeeRequest",
    "GetFeeRequestTypedDict",
    "GetInvoiceLineItemsRequest",
    "GetInvoiceLineItemsRequestTypedDict",
    "GetInvoiceRequest",
    "GetInvoiceRequestTypedDict",
    "GetPaymentRequest",
    "GetPaymentRequestTypedDict",
    "GetPlanRequest",
    "GetPlanRequestTypedDict",
    "GetPriceRequest",
    "GetPriceRequestTypedDict",
    "GetProductRequest",
    "GetProductRequestTypedDict",
    "GetRevenueRequest",
    "GetRevenueRequestTypedDict",
    "GetSourceRequest",
    "GetSourceRequestTypedDict",
    "GetSourceRuleRequest",
    "GetSourceRuleRequestTypedDict",
    "GetSubscriptionRequest",
    "GetSubscriptionRequestTypedDict",
    "GetTestClockRequest",
    "GetTestClockRequestTypedDict",
    "GetUsageEventRequest",
    "GetUsageEventRequestTypedDict",
    "GetUserRequest",
    "GetUserRequestTypedDict",
    "Grant",
    "GrantObject",
    "GrantTypedDict",
    "GroupBy",
    "GroupInvoiceSummary",
    "GroupInvoiceSummaryTypedDict",
    "GroupTrendEntry",
    "GroupTrendEntryTypedDict",
    "GroupedValue",
    "GroupedValueTypedDict",
    "IngestEventRequest",
    "IngestEventRequestTypedDict",
    "Invoice",
    "InvoiceCategorySummary",
    "InvoiceCategorySummaryTypedDict",
    "InvoiceLineItem",
    "InvoiceLineItemLineItemType",
    "InvoiceLineItemTypedDict",
    "InvoiceLineItemsResponse",
    "InvoiceLineItemsResponseTypedDict",
    "InvoiceObject",
    "InvoiceStatus",
    "InvoiceSummary",
    "InvoiceSummaryTypedDict",
    "InvoiceTypedDict",
    "IssueEntitlementRequest",
    "IssueEntitlementRequestTypedDict",
    "Item",
    "ItemTypedDict",
    "LineItem",
    "LineItemPaymentTerm",
    "LineItemStatus",
    "LineItemType",
    "LineItemTypedDict",
    "LineItems",
    "LineItemsResponse",
    "LineItemsResponseObject",
    "LineItemsResponseTypedDict",
    "LineItemsSummary",
    "LineItemsSummaryTypedDict",
    "LineItemsTypedDict",
    "ListAvailablePlansPagination",
    "ListAvailablePlansPaginationTypedDict",
    "ListAvailablePlansRequest",
    "ListAvailablePlansRequestTypedDict",
    "ListAvailablePlansResponse",
    "ListAvailablePlansResponseTypedDict",
    "ListBillableMetricsObject",
    "ListBillableMetricsPagination",
    "ListBillableMetricsPaginationTypedDict",
    "ListBillableMetricsRequest",
    "ListBillableMetricsRequestTypedDict",
    "ListBillableMetricsResponse",
    "ListBillableMetricsResponseTypedDict",
    "ListCustomersObject",
    "ListCustomersRequest",
    "ListCustomersRequestTypedDict",
    "ListCustomersResponse",
    "ListCustomersResponseTypedDict",
    "ListDisputesObject",
    "ListDisputesPagination",
    "ListDisputesPaginationTypedDict",
    "ListDisputesRequest",
    "ListDisputesRequestTypedDict",
    "ListDisputesResponse",
    "ListDisputesResponseTypedDict",
    "ListDisputesStatus",
    "ListEntitlementGrantsObject",
    "ListEntitlementGrantsRequest",
    "ListEntitlementGrantsRequestTypedDict",
    "ListEntitlementGrantsResponse",
    "ListEntitlementGrantsResponseTypedDict",
    "ListEntitlementsObject",
    "ListEntitlementsRequest",
    "ListEntitlementsRequestTypedDict",
    "ListEntitlementsResponse",
    "ListEntitlementsResponseTypedDict",
    "ListFeaturesObject",
    "ListFeaturesPagination",
    "ListFeaturesPaginationTypedDict",
    "ListFeaturesRequest",
    "ListFeaturesRequestTypedDict",
    "ListFeaturesResponse",
    "ListFeaturesResponseTypedDict",
    "ListFeesObject",
    "ListFeesPagination",
    "ListFeesPaginationTypedDict",
    "ListFeesRequest",
    "ListFeesRequestTypedDict",
    "ListFeesResponse",
    "ListFeesResponseTypedDict",
    "ListInvoicesObject",
    "ListInvoicesRequest",
    "ListInvoicesRequestTypedDict",
    "ListInvoicesResponse",
    "ListInvoicesResponseTypedDict",
    "ListInvoicesStatus",
    "ListLineItemsRequest",
    "ListLineItemsRequestTypedDict",
    "ListLineItemsStatus",
    "ListPaymentsObject",
    "ListPaymentsRequest",
    "ListPaymentsRequestTypedDict",
    "ListPaymentsResponse",
    "ListPaymentsResponseTypedDict",
    "ListPaymentsStatus",
    "ListPlansPagination",
    "ListPlansPaginationTypedDict",
    "ListPlansRequest",
    "ListPlansRequestTypedDict",
    "ListPlansResponse",
    "ListPlansResponseTypedDict",
    "ListPricesPagination",
    "ListPricesPaginationTypedDict",
    "ListPricesRequest",
    "ListPricesRequestTypedDict",
    "ListPricesResponse",
    "ListPricesResponseTypedDict",
    "ListProductsRequest",
    "ListProductsRequestTypedDict",
    "ListProductsResponse",
    "ListProductsResponseTypedDict",
    "ListSourceEventsRequest",
    "ListSourceEventsRequestTypedDict",
    "ListSourceEventsResponse",
    "ListSourceEventsResponseTypedDict",
    "ListSourceEventsStatus",
    "ListSourceRulesRequest",
    "ListSourceRulesRequestTypedDict",
    "ListSourceRulesResponse",
    "ListSourceRulesResponseTypedDict",
    "ListSourcesRequest",
    "ListSourcesRequestTypedDict",
    "ListSourcesResponse",
    "ListSourcesResponseTypedDict",
    "ListSubscriptionsRequest",
    "ListSubscriptionsRequestTypedDict",
    "ListSubscriptionsResponse",
    "ListSubscriptionsResponseTypedDict",
    "ListSubscriptionsStatus",
    "ListTestClocksPagination",
    "ListTestClocksPaginationTypedDict",
    "ListTestClocksRequest",
    "ListTestClocksRequestTypedDict",
    "ListTestClocksResponse",
    "ListTestClocksResponseTypedDict",
    "ListUsageEventsObject",
    "ListUsageEventsRequest",
    "ListUsageEventsRequestTypedDict",
    "ListUsageEventsResponse",
    "ListUsageEventsResponseTypedDict",
    "Metadata",
    "MetadataTypedDict",
    "MeteredEntitlementDetail",
    "MeteredEntitlementDetailObject",
    "MeteredEntitlementDetailStatus",
    "MeteredEntitlementDetailTypedDict",
    "NextActionAt",
    "OffsetPagination",
    "OffsetPaginationTypedDict",
    "Operator",
    "Parameters",
    "ParametersTypedDict",
    "Payment",
    "PaymentLineItem",
    "PaymentLineItemTypedDict",
    "PaymentObject",
    "PaymentPaid",
    "PaymentPaidTypedDict",
    "PaymentPending",
    "PaymentPendingTypedDict",
    "PaymentSession",
    "PaymentSessionTypedDict",
    "PaymentStatus",
    "PaymentSummary",
    "PaymentSummaryTypedDict",
    "PaymentTypedDict",
    "PaymentUnion",
    "PaymentUnionTypedDict",
    "Plan",
    "PlanObject",
    "PlanPaymentTerm",
    "PlanPaymentTermTypedDict",
    "PlanTypedDict",
    "Price",
    "PriceFeature",
    "PriceFeatureFeature",
    "PriceFeatureFeatureTypedDict",
    "PriceFeatureInput",
    "PriceFeatureInputTypedDict",
    "PriceFeatureType",
    "PriceFeatureTypedDict",
    "PriceModel",
    "PriceObject",
    "PricePaymentTerm",
    "PriceProperties1",
    "PriceProperties1TypedDict",
    "PriceProperties2",
    "PriceProperties2TypedDict",
    "PriceProperties3",
    "PriceProperties3TypedDict",
    "PriceProperties4",
    "PriceProperties4TypedDict",
    "PricePropertiesUnion",
    "PricePropertiesUnionTypedDict",
    "PriceTypedDict",
    "ProcessingMode",
    "Product",
    "ProductObject",
    "ProductTypedDict",
    "Properties",
    "PropertiesTypedDict",
    "PurchaseEntitlementGrantRequest",
    "PurchaseEntitlementGrantRequestTypedDict",
    "PurchaseGrantRequest",
    "PurchaseGrantRequestTypedDict",
    "PurchaseGrantResponse",
    "PurchaseGrantResponseObject",
    "PurchaseGrantResponseTypedDict",
    "RedirectUrls",
    "RedirectUrlsTypedDict",
    "Refund",
    "RefundTypedDict",
    "RefundUsageEventRequest",
    "RefundUsageEventRequestBody",
    "RefundUsageEventRequestBodyTypedDict",
    "RefundUsageEventRequestTypedDict",
    "RejectSourceEventRequest",
    "RejectSourceEventRequestTypedDict",
    "RequestBody1",
    "RequestBody1TypedDict",
    "RequestBody2",
    "RequestBody2TypedDict",
    "RevenueSummaryResponse",
    "RevenueSummaryResponseTypedDict",
    "RevenueTrendBucket",
    "RevenueTrendBucketTypedDict",
    "RuleCondition",
    "RuleConditionType",
    "RuleConditionTypedDict",
    "SchemasEntitlement",
    "SchemasEntitlementObject",
    "SchemasEntitlementStatus",
    "SchemasEntitlementTypedDict",
    "SchemasPrice",
    "SchemasPriceModel",
    "SchemasPriceObject",
    "SchemasPricePaymentTerm",
    "SchemasPriceTypedDict",
    "Security",
    "SecurityTypedDict",
    "Source",
    "SourceEvent",
    "SourceEventStatus",
    "SourceEventTypedDict",
    "SourceObject",
    "SourceRule",
    "SourceRuleTypedDict",
    "SourceType",
    "SourceTypedDict",
    "State",
    "StaticEntitlementDetail",
    "StaticEntitlementDetailObject",
    "StaticEntitlementDetailStatus",
    "StaticEntitlementDetailTypedDict",
    "Subscription",
    "SubscriptionObject",
    "SubscriptionPortal",
    "SubscriptionPortalObject",
    "SubscriptionPortalTypedDict",
    "SubscriptionStatusEnum",
    "SubscriptionTypedDict",
    "Summary",
    "SummaryTypedDict",
    "Tax",
    "TaxBehavior",
    "TaxRates",
    "TaxRatesTypedDict",
    "TaxTypedDict",
    "TerminateSubscriptionRequest",
    "TerminateSubscriptionRequestBody",
    "TerminateSubscriptionRequestBodyTypedDict",
    "TerminateSubscriptionRequestTypedDict",
    "TestClock",
    "TestClockTypedDict",
    "To",
    "ToTypedDict",
    "UnknownEntitlementDetail",
    "UnknownPaymentUnion",
    "UpdateBillableMetricRequest",
    "UpdateBillableMetricRequestBody",
    "UpdateBillableMetricRequestBodyTypedDict",
    "UpdateBillableMetricRequestTypedDict",
    "UpdateCustomerRequest",
    "UpdateCustomerRequestBody",
    "UpdateCustomerRequestBodyTypedDict",
    "UpdateCustomerRequestTypedDict",
    "UpdateFeatureRequest",
    "UpdateFeatureRequestBody",
    "UpdateFeatureRequestBodyTypedDict",
    "UpdateFeatureRequestTypedDict",
    "UpdateFeatureType",
    "UpdateFeeRequest",
    "UpdateFeeRequestBody",
    "UpdateFeeRequestBodyTypedDict",
    "UpdateFeeRequestTypedDict",
    "UpdatePlanRequest",
    "UpdatePlanRequestBody",
    "UpdatePlanRequestBodyTypedDict",
    "UpdatePlanRequestTypedDict",
    "UpdatePlanTaxBehavior",
    "UpdatePriceModel",
    "UpdatePricePaymentTerm",
    "UpdatePriceRequest",
    "UpdatePriceRequestBody",
    "UpdatePriceRequestBodyTypedDict",
    "UpdatePriceRequestTypedDict",
    "UpdateProductRequest",
    "UpdateProductRequestBody",
    "UpdateProductRequestBodyTypedDict",
    "UpdateProductRequestTypedDict",
    "UpdateRuleRequest",
    "UpdateRuleRequestTypedDict",
    "UpdateSourceProcessingMode",
    "UpdateSourceRequest",
    "UpdateSourceRequestBody",
    "UpdateSourceRequestBodyTypedDict",
    "UpdateSourceRequestTypedDict",
    "UpdateSourceRuleRequest",
    "UpdateSourceRuleRequestTypedDict",
    "UpdateUserRequest",
    "UpdateUserRequestBody",
    "UpdateUserRequestBodyTypedDict",
    "UpdateUserRequestTypedDict",
    "UpdateUserType",
    "UsageEvent",
    "UsageEventObject",
    "UsageEventProperty",
    "UsageEventPropertyTypedDict",
    "UsageEventTypedDict",
    "UsagePeriod",
    "UsagePeriodTypedDict",
    "UsageResponse",
    "UsageResponseObject",
    "UsageResponseTypedDict",
    "User",
    "UserObject",
    "UserOrganization",
    "UserOrganizationTypedDict",
    "UserType",
    "UserTypedDict",
    "ValidTaxAddress",
    "ValidTaxAddressTypedDict",
    "Value",
    "ValueTypedDict",
    "ValueUnion",
    "ValueUnionTypedDict",
    "VoidEntitlementGrantRequest",
    "VoidEntitlementGrantRequestTypedDict",
    "WindowSize",
    "WindowedValue",
    "WindowedValueTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "ActiveEntitlement": ".activeentitlement",
    "ActiveEntitlementObject": ".activeentitlement",
    "ActiveEntitlementTypedDict": ".activeentitlement",
    "Address": ".address",
    "AddressTypedDict": ".address",
    "AdvanceTestClockRequest": ".advancetestclockop",
    "AdvanceTestClockRequestBody": ".advancetestclockop",
    "AdvanceTestClockRequestBodyTypedDict": ".advancetestclockop",
    "AdvanceTestClockRequestTypedDict": ".advancetestclockop",
    "RequestBody1": ".advancetestclockop",
    "RequestBody1TypedDict": ".advancetestclockop",
    "RequestBody2": ".advancetestclockop",
    "RequestBody2TypedDict": ".advancetestclockop",
    "ApproveSourceEventRequest": ".approvesourceeventop",
    "ApproveSourceEventRequestTypedDict": ".approvesourceeventop",
    "BatchCreateUsageEventsRequest": ".batchcreateusageeventsop",
    "BatchCreateUsageEventsRequestTypedDict": ".batchcreateusageeventsop",
    "BatchUsageEventResponse": ".batchusageeventresponse",
    "BatchUsageEventResponseObject": ".batchusageeventresponse",
    "BatchUsageEventResponseTypedDict": ".batchusageeventresponse",
    "Failed": ".batchusageeventresponse",
    "FailedTypedDict": ".batchusageeventresponse",
    "Summary": ".batchusageeventresponse",
    "SummaryTypedDict": ".batchusageeventresponse",
    "Aggregation": ".billablemetric",
    "BillableMetric": ".billablemetric",
    "BillableMetricObject": ".billablemetric",
    "BillableMetricTypedDict": ".billablemetric",
    "BooleanEntitlementDetail": ".booleanentitlementdetail",
    "BooleanEntitlementDetailObject": ".booleanentitlementdetail",
    "BooleanEntitlementDetailStatus": ".booleanentitlementdetail",
    "BooleanEntitlementDetailTypedDict": ".booleanentitlementdetail",
    "BulkApproveSourceEventsDetails": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsDetailsTypedDict": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsFailed": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsFailedTypedDict": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsRequest": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsRequestBody": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsRequestBodyTypedDict": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsRequestTypedDict": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsResponse": ".bulkapprovesourceeventsop",
    "BulkApproveSourceEventsResponseTypedDict": ".bulkapprovesourceeventsop",
    "BulkRejectSourceEventsRequest": ".bulkrejectsourceeventsop",
    "BulkRejectSourceEventsRequestBody": ".bulkrejectsourceeventsop",
    "BulkRejectSourceEventsRequestBodyTypedDict": ".bulkrejectsourceeventsop",
    "BulkRejectSourceEventsRequestTypedDict": ".bulkrejectsourceeventsop",
    "BulkRejectSourceEventsResponse": ".bulkrejectsourceeventsop",
    "BulkRejectSourceEventsResponseTypedDict": ".bulkrejectsourceeventsop",
    "CreateBillableMetricAggregation": ".createbillablemetricop",
    "CreateBillableMetricRequest": ".createbillablemetricop",
    "CreateBillableMetricRequestTypedDict": ".createbillablemetricop",
    "Consumer": ".createcustomerop",
    "ConsumerTypedDict": ".createcustomerop",
    "CreateCustomerRequest": ".createcustomerop",
    "CreateCustomerRequestTypedDict": ".createcustomerop",
    "CreateCustomerResponse": ".createcustomerop",
    "CreateCustomerResponseBody1": ".createcustomerop",
    "CreateCustomerResponseBody1TypedDict": ".createcustomerop",
    "CreateCustomerResponseBody2": ".createcustomerop",
    "CreateCustomerResponseBody2TypedDict": ".createcustomerop",
    "CreateCustomerResponseTypedDict": ".createcustomerop",
    "CreateDisputeRequest": ".createdisputerequest",
    "CreateDisputeRequestTypedDict": ".createdisputerequest",
    "CreateEntitlementGrantRequest": ".createentitlementgrantop",
    "CreateEntitlementGrantRequestTypedDict": ".createentitlementgrantop",
    "CreateEntitlementRequest": ".createentitlementrequest",
    "CreateEntitlementRequestTypedDict": ".createentitlementrequest",
    "EntitlementDatum": ".createentitlementrequest",
    "EntitlementDatumTypedDict": ".createentitlementrequest",
    "CreateFeatureRequest": ".createfeatureop",
    "CreateFeatureRequestTypedDict": ".createfeatureop",
    "CreateFeatureType": ".createfeatureop",
    "CreateFeeRequest": ".createfeeop",
    "CreateFeeRequestTypedDict": ".createfeeop",
    "CreateGrantRequest": ".creategrantrequest",
    "CreateGrantRequestTypedDict": ".creategrantrequest",
    "CreateLineItemRequest": ".createlineitemop",
    "CreateLineItemRequestTypedDict": ".createlineitemop",
    "CreateManualLineItemRequest": ".createmanuallineitemrequest",
    "CreateManualLineItemRequestTypedDict": ".createmanuallineitemrequest",
    "CreatePaymentLineItem": ".createpaymentop",
    "CreatePaymentLineItemTypedDict": ".createpaymentop",
    "CreatePaymentRequest": ".createpaymentop",
    "CreatePaymentRequestTypedDict": ".createpaymentop",
    "Currency": ".createpaymentop",
    "BillingVersion": ".createplanop",
    "CreatePlanRequest": ".createplanop",
    "CreatePlanRequestTypedDict": ".createplanop",
    "CreatePlanTaxBehavior": ".createplanop",
    "CreatePriceModel": ".createpriceop",
    "CreatePricePaymentTerm": ".createpriceop",
    "CreatePriceRequest": ".createpriceop",
    "CreatePriceRequestTypedDict": ".createpriceop",
    "CreateProductRequest": ".createproductop",
    "CreateProductRequestTypedDict": ".createproductop",
    "Metadata": ".createproductop",
    "MetadataTypedDict": ".createproductop",
    "CreateRuleRequest": ".createrulerequest",
    "CreateRuleRequestTypedDict": ".createrulerequest",
    "CreateSourceProcessingMode": ".createsourceop",
    "CreateSourceRequest": ".createsourceop",
    "CreateSourceRequestTypedDict": ".createsourceop",
    "CreateSourceType": ".createsourceop",
    "CreateSubscriptionAddress": ".createsubscriptionop",
    "CreateSubscriptionAddressTypedDict": ".createsubscriptionop",
    "CreateSubscriptionCustomer": ".createsubscriptionop",
    "CreateSubscriptionCustomerTypedDict": ".createsubscriptionop",
    "CreateSubscriptionRequest": ".createsubscriptionop",
    "CreateSubscriptionRequestTypedDict": ".createsubscriptionop",
    "RedirectUrls": ".createsubscriptionop",
    "RedirectUrlsTypedDict": ".createsubscriptionop",
    "CreateTestClockRequest": ".createtestclockop",
    "CreateTestClockRequestTypedDict": ".createtestclockop",
    "CreateUsageEventRequest": ".createusageeventrequest",
    "CreateUsageEventRequestProperty": ".createusageeventrequest",
    "CreateUsageEventRequestPropertyTypedDict": ".createusageeventrequest",
    "CreateUsageEventRequestTypedDict": ".createusageeventrequest",
    "Customer": ".customer",
    "CustomerObject": ".customer",
    "CustomerOrganization": ".customer",
    "CustomerOrganizationTypedDict": ".customer",
    "CustomerTypedDict": ".customer",
    "DeleteCustomerBlocker": ".deletecustomerop",
    "DeleteCustomerBlockerTypedDict": ".deletecustomerop",
    "DeleteCustomerDetails": ".deletecustomerop",
    "DeleteCustomerDetailsTypedDict": ".deletecustomerop",
    "DeleteCustomerError": ".deletecustomerop",
    "DeleteCustomerRequest": ".deletecustomerop",
    "DeleteCustomerRequestTypedDict": ".deletecustomerop",
    "DeleteCustomerType": ".deletecustomerop",
    "Item": ".deletecustomerop",
    "ItemTypedDict": ".deletecustomerop",
    "DeleteFeatureRequest": ".deletefeatureop",
    "DeleteFeatureRequestTypedDict": ".deletefeatureop",
    "DeleteFeeBlocker": ".deletefeeop",
    "DeleteFeeBlockerTypedDict": ".deletefeeop",
    "DeleteFeeDetails": ".deletefeeop",
    "DeleteFeeDetailsTypedDict": ".deletefeeop",
    "DeleteFeeRequest": ".deletefeeop",
    "DeleteFeeRequestTypedDict": ".deletefeeop",
    "DeletePriceRequest": ".deletepriceop",
    "DeletePriceRequestTypedDict": ".deletepriceop",
    "DeleteSourceRuleRequest": ".deletesourceruleop",
    "DeleteSourceRuleRequestTypedDict": ".deletesourceruleop",
    "DeleteTestClockRequest": ".deletetestclockop",
    "DeleteTestClockRequestTypedDict": ".deletetestclockop",
    "Dispute": ".dispute",
    "DisputeObject": ".dispute",
    "DisputeStatus": ".dispute",
    "DisputeTypedDict": ".dispute",
    "DisputeWithCustomer": ".disputewithcustomer",
    "DisputeWithCustomerCustomer": ".disputewithcustomer",
    "DisputeWithCustomerCustomerTypedDict": ".disputewithcustomer",
    "DisputeWithCustomerObject": ".disputewithcustomer",
    "DisputeWithCustomerStatus": ".disputewithcustomer",
    "DisputeWithCustomerTypedDict": ".disputewithcustomer",
    "Entitlement": ".entitlement",
    "EntitlementBillableMetric": ".entitlement",
    "EntitlementBillableMetricTypedDict": ".entitlement",
    "EntitlementObject": ".entitlement",
    "EntitlementTypedDict": ".entitlement",
    "EntitlementAccessResult": ".entitlementaccessresult",
    "EntitlementAccessResultStatus": ".entitlementaccessresult",
    "EntitlementAccessResultTypedDict": ".entitlementaccessresult",
    "FeatureTypeEnum": ".entitlementaccessresult",
    "EntitlementDetail": ".entitlementdetail",
    "EntitlementDetailTypedDict": ".entitlementdetail",
    "UnknownEntitlementDetail": ".entitlementdetail",
    "EntitlementTemplate": ".entitlementtemplate",
    "EntitlementTemplateBoolean": ".entitlementtemplate",
    "EntitlementTemplateBooleanTypedDict": ".entitlementtemplate",
    "EntitlementTemplateMetered": ".entitlementtemplate",
    "EntitlementTemplateMeteredTypedDict": ".entitlementtemplate",
    "EntitlementTemplateStatic": ".entitlementtemplate",
    "EntitlementTemplateStaticTypedDict": ".entitlementtemplate",
    "EntitlementTemplateTypedDict": ".entitlementtemplate",
    "UsagePeriod": ".entitlementtemplate",
    "UsagePeriodTypedDict": ".entitlementtemplate",
    "EventResponse": ".eventresponse",
    "EventResponseObject": ".eventresponse",
    "EventResponseTypedDict": ".eventresponse",
    "Feature": ".feature",
    "FeatureObject": ".feature",
    "FeatureType": ".feature",
    "FeatureTypedDict": ".feature",
    "Fee": ".fee",
    "FeeObject": ".fee",
    "FeeTypedDict": ".fee",
    "FeePrice": ".feeprice",
    "FeePriceModel": ".feeprice",
    "FeePricePaymentTerm": ".feeprice",
    "FeePriceTypedDict": ".feeprice",
    "Properties": ".feeprice",
    "PropertiesTypedDict": ".feeprice",
    "GeneratePortalLinkRequest": ".generateportallinkop",
    "GeneratePortalLinkRequestBody": ".generateportallinkop",
    "GeneratePortalLinkRequestBodyTypedDict": ".generateportallinkop",
    "GeneratePortalLinkRequestTypedDict": ".generateportallinkop",
    "GetActiveEntitlementsObject": ".getactiveentitlementsop",
    "GetActiveEntitlementsPagination": ".getactiveentitlementsop",
    "GetActiveEntitlementsPaginationTypedDict": ".getactiveentitlementsop",
    "GetActiveEntitlementsRequest": ".getactiveentitlementsop",
    "GetActiveEntitlementsRequestTypedDict": ".getactiveentitlementsop",
    "GetActiveEntitlementsResponse": ".getactiveentitlementsop",
    "GetActiveEntitlementsResponseTypedDict": ".getactiveentitlementsop",
    "GetBillableMetricMeterRequest": ".getbillablemetricmeterop",
    "GetBillableMetricMeterRequestTypedDict": ".getbillablemetricmeterop",
    "WindowSize": ".getbillablemetricmeterop",
    "GetBillableMetricRequest": ".getbillablemetricop",
    "GetBillableMetricRequestTypedDict": ".getbillablemetricop",
    "GetCustomerRequest": ".getcustomerop",
    "GetCustomerRequestTypedDict": ".getcustomerop",
    "GetEntitlementGrantRequest": ".getentitlementgrantop",
    "GetEntitlementGrantRequestTypedDict": ".getentitlementgrantop",
    "GetEntitlementRequest": ".getentitlementop",
    "GetEntitlementRequestTypedDict": ".getentitlementop",
    "GetFeatureRequest": ".getfeatureop",
    "GetFeatureRequestTypedDict": ".getfeatureop",
    "GetFeeRequest": ".getfeeop",
    "GetFeeRequestTypedDict": ".getfeeop",
    "GetFeePriceRequest": ".getfeepriceop",
    "GetFeePriceRequestTypedDict": ".getfeepriceop",
    "GetInvoiceLineItemsRequest": ".getinvoicelineitemsop",
    "GetInvoiceLineItemsRequestTypedDict": ".getinvoicelineitemsop",
    "GetInvoiceRequest": ".getinvoiceop",
    "GetInvoiceRequestTypedDict": ".getinvoiceop",
    "GetPaymentRequest": ".getpaymentop",
    "GetPaymentRequestTypedDict": ".getpaymentop",
    "GetPlanRequest": ".getplanop",
    "GetPlanRequestTypedDict": ".getplanop",
    "GetPriceRequest": ".getpriceop",
    "GetPriceRequestTypedDict": ".getpriceop",
    "GetProductRequest": ".getproductop",
    "GetProductRequestTypedDict": ".getproductop",
    "BucketWidth": ".getrevenueop",
    "GetRevenueRequest": ".getrevenueop",
    "GetRevenueRequestTypedDict": ".getrevenueop",
    "GroupBy": ".getrevenueop",
    "GetSourceRequest": ".getsourceop",
    "GetSourceRequestTypedDict": ".getsourceop",
    "GetSourceRuleRequest": ".getsourceruleop",
    "GetSourceRuleRequestTypedDict": ".getsourceruleop",
    "GetSubscriptionRequest": ".getsubscriptionop",
    "GetSubscriptionRequestTypedDict": ".getsubscriptionop",
    "GetTestClockRequest": ".gettestclockop",
    "GetTestClockRequestTypedDict": ".gettestclockop",
    "GetUsageEventRequest": ".getusageeventop",
    "GetUsageEventRequestTypedDict": ".getusageeventop",
    "GetUserRequest": ".getuserop",
    "GetUserRequestTypedDict": ".getuserop",
    "Grant": ".grant",
    "GrantObject": ".grant",
    "GrantTypedDict": ".grant",
    "GroupInvoiceSummary": ".groupinvoicesummary",
    "GroupInvoiceSummaryTypedDict": ".groupinvoicesummary",
    "GroupTrendEntry": ".grouptrendentry",
    "GroupTrendEntryTypedDict": ".grouptrendentry",
    "IngestEventRequest": ".ingesteventop",
    "IngestEventRequestTypedDict": ".ingesteventop",
    "Invoice": ".invoice",
    "InvoiceObject": ".invoice",
    "InvoiceStatus": ".invoice",
    "InvoiceTypedDict": ".invoice",
    "LineItems": ".invoice",
    "LineItemsTypedDict": ".invoice",
    "Tax": ".invoice",
    "TaxTypedDict": ".invoice",
    "InvoiceCategorySummary": ".invoicecategorysummary",
    "InvoiceCategorySummaryTypedDict": ".invoicecategorysummary",
    "EventType": ".invoicelineitem",
    "InvoiceLineItem": ".invoicelineitem",
    "InvoiceLineItemLineItemType": ".invoicelineitem",
    "InvoiceLineItemTypedDict": ".invoicelineitem",
    "InvoiceLineItemsResponse": ".invoicelineitemsresponse",
    "InvoiceLineItemsResponseTypedDict": ".invoicelineitemsresponse",
    "InvoiceSummary": ".invoicesummary",
    "InvoiceSummaryTypedDict": ".invoicesummary",
    "IssueEntitlementRequest": ".issueentitlementrequest",
    "IssueEntitlementRequestTypedDict": ".issueentitlementrequest",
    "LineItem": ".lineitem",
    "LineItemPaymentTerm": ".lineitem",
    "LineItemStatus": ".lineitem",
    "LineItemType": ".lineitem",
    "LineItemTypedDict": ".lineitem",
    "LineItemsResponse": ".lineitemsresponse",
    "LineItemsResponseObject": ".lineitemsresponse",
    "LineItemsResponseTypedDict": ".lineitemsresponse",
    "LineItemsSummary": ".lineitemssummary",
    "LineItemsSummaryTypedDict": ".lineitemssummary",
    "ListAvailablePlansPagination": ".listavailableplansop",
    "ListAvailablePlansPaginationTypedDict": ".listavailableplansop",
    "ListAvailablePlansRequest": ".listavailableplansop",
    "ListAvailablePlansRequestTypedDict": ".listavailableplansop",
    "ListAvailablePlansResponse": ".listavailableplansop",
    "ListAvailablePlansResponseTypedDict": ".listavailableplansop",
    "ListBillableMetricsObject": ".listbillablemetricsop",
    "ListBillableMetricsPagination": ".listbillablemetricsop",
    "ListBillableMetricsPaginationTypedDict": ".listbillablemetricsop",
    "ListBillableMetricsRequest": ".listbillablemetricsop",
    "ListBillableMetricsRequestTypedDict": ".listbillablemetricsop",
    "ListBillableMetricsResponse": ".listbillablemetricsop",
    "ListBillableMetricsResponseTypedDict": ".listbillablemetricsop",
    "ListCustomersObject": ".listcustomersop",
    "ListCustomersRequest": ".listcustomersop",
    "ListCustomersRequestTypedDict": ".listcustomersop",
    "ListCustomersResponse": ".listcustomersop",
    "ListCustomersResponseTypedDict": ".listcustomersop",
    "ListDisputesObject": ".listdisputesop",
    "ListDisputesPagination": ".listdisputesop",
    "ListDisputesPaginationTypedDict": ".listdisputesop",
    "ListDisputesRequest": ".listdisputesop",
    "ListDisputesRequestTypedDict": ".listdisputesop",
    "ListDisputesResponse": ".listdisputesop",
    "ListDisputesResponseTypedDict": ".listdisputesop",
    "ListDisputesStatus": ".listdisputesop",
    "ListEntitlementGrantsObject": ".listentitlementgrantsop",
    "ListEntitlementGrantsRequest": ".listentitlementgrantsop",
    "ListEntitlementGrantsRequestTypedDict": ".listentitlementgrantsop",
    "ListEntitlementGrantsResponse": ".listentitlementgrantsop",
    "ListEntitlementGrantsResponseTypedDict": ".listentitlementgrantsop",
    "ListEntitlementsObject": ".listentitlementsop",
    "ListEntitlementsRequest": ".listentitlementsop",
    "ListEntitlementsRequestTypedDict": ".listentitlementsop",
    "ListEntitlementsResponse": ".listentitlementsop",
    "ListEntitlementsResponseTypedDict": ".listentitlementsop",
    "ListFeaturesObject": ".listfeaturesop",
    "ListFeaturesPagination": ".listfeaturesop",
    "ListFeaturesPaginationTypedDict": ".listfeaturesop",
    "ListFeaturesRequest": ".listfeaturesop",
    "ListFeaturesRequestTypedDict": ".listfeaturesop",
    "ListFeaturesResponse": ".listfeaturesop",
    "ListFeaturesResponseTypedDict": ".listfeaturesop",
    "ListFeesObject": ".listfeesop",
    "ListFeesPagination": ".listfeesop",
    "ListFeesPaginationTypedDict": ".listfeesop",
    "ListFeesRequest": ".listfeesop",
    "ListFeesRequestTypedDict": ".listfeesop",
    "ListFeesResponse": ".listfeesop",
    "ListFeesResponseTypedDict": ".listfeesop",
    "ListInvoicesObject": ".listinvoicesop",
    "ListInvoicesRequest": ".listinvoicesop",
    "ListInvoicesRequestTypedDict": ".listinvoicesop",
    "ListInvoicesResponse": ".listinvoicesop",
    "ListInvoicesResponseTypedDict": ".listinvoicesop",
    "ListInvoicesStatus": ".listinvoicesop",
    "NextActionAt": ".listinvoicesop",
    "ListLineItemsRequest": ".listlineitemsop",
    "ListLineItemsRequestTypedDict": ".listlineitemsop",
    "ListLineItemsStatus": ".listlineitemsop",
    "ListPaymentsObject": ".listpaymentsop",
    "ListPaymentsRequest": ".listpaymentsop",
    "ListPaymentsRequestTypedDict": ".listpaymentsop",
    "ListPaymentsResponse": ".listpaymentsop",
    "ListPaymentsResponseTypedDict": ".listpaymentsop",
    "ListPaymentsStatus": ".listpaymentsop",
    "ListPlansPagination": ".listplansop",
    "ListPlansPaginationTypedDict": ".listplansop",
    "ListPlansRequest": ".listplansop",
    "ListPlansRequestTypedDict": ".listplansop",
    "ListPlansResponse": ".listplansop",
    "ListPlansResponseTypedDict": ".listplansop",
    "ListPricesPagination": ".listpricesop",
    "ListPricesPaginationTypedDict": ".listpricesop",
    "ListPricesRequest": ".listpricesop",
    "ListPricesRequestTypedDict": ".listpricesop",
    "ListPricesResponse": ".listpricesop",
    "ListPricesResponseTypedDict": ".listpricesop",
    "ListProductsRequest": ".listproductsop",
    "ListProductsRequestTypedDict": ".listproductsop",
    "ListProductsResponse": ".listproductsop",
    "ListProductsResponseTypedDict": ".listproductsop",
    "ListSourceEventsRequest": ".listsourceeventsop",
    "ListSourceEventsRequestTypedDict": ".listsourceeventsop",
    "ListSourceEventsResponse": ".listsourceeventsop",
    "ListSourceEventsResponseTypedDict": ".listsourceeventsop",
    "ListSourceEventsStatus": ".listsourceeventsop",
    "ListSourceRulesRequest": ".listsourcerulesop",
    "ListSourceRulesRequestTypedDict": ".listsourcerulesop",
    "ListSourceRulesResponse": ".listsourcerulesop",
    "ListSourceRulesResponseTypedDict": ".listsourcerulesop",
    "ListSourcesRequest": ".listsourcesop",
    "ListSourcesRequestTypedDict": ".listsourcesop",
    "ListSourcesResponse": ".listsourcesop",
    "ListSourcesResponseTypedDict": ".listsourcesop",
    "ListSubscriptionsRequest": ".listsubscriptionsop",
    "ListSubscriptionsRequestTypedDict": ".listsubscriptionsop",
    "ListSubscriptionsResponse": ".listsubscriptionsop",
    "ListSubscriptionsResponseTypedDict": ".listsubscriptionsop",
    "ListSubscriptionsStatus": ".listsubscriptionsop",
    "ListTestClocksPagination": ".listtestclocksop",
    "ListTestClocksPaginationTypedDict": ".listtestclocksop",
    "ListTestClocksRequest": ".listtestclocksop",
    "ListTestClocksRequestTypedDict": ".listtestclocksop",
    "ListTestClocksResponse": ".listtestclocksop",
    "ListTestClocksResponseTypedDict": ".listtestclocksop",
    "ListUsageEventsObject": ".listusageeventsop",
    "ListUsageEventsRequest": ".listusageeventsop",
    "ListUsageEventsRequestTypedDict": ".listusageeventsop",
    "ListUsageEventsResponse": ".listusageeventsop",
    "ListUsageEventsResponseTypedDict": ".listusageeventsop",
    "MeteredEntitlementDetail": ".meteredentitlementdetail",
    "MeteredEntitlementDetailObject": ".meteredentitlementdetail",
    "MeteredEntitlementDetailStatus": ".meteredentitlementdetail",
    "MeteredEntitlementDetailTypedDict": ".meteredentitlementdetail",
    "OffsetPagination": ".offsetpagination",
    "OffsetPaginationTypedDict": ".offsetpagination",
    "Payment": ".payment",
    "PaymentLineItem": ".payment",
    "PaymentLineItemTypedDict": ".payment",
    "PaymentObject": ".payment",
    "PaymentStatus": ".payment",
    "PaymentTypedDict": ".payment",
    "PaymentSummary": ".paymentsummary",
    "PaymentSummaryTypedDict": ".paymentsummary",
    "Plan": ".plan",
    "PlanObject": ".plan",
    "PlanPaymentTerm": ".plan",
    "PlanPaymentTermTypedDict": ".plan",
    "PlanTypedDict": ".plan",
    "TaxBehavior": ".plan",
    "Price": ".price",
    "PriceModel": ".price",
    "PriceObject": ".price",
    "PricePaymentTerm": ".price",
    "PriceTypedDict": ".price",
    "PriceFeature": ".pricefeature",
    "PriceFeatureFeature": ".pricefeature",
    "PriceFeatureFeatureTypedDict": ".pricefeature",
    "PriceFeatureType": ".pricefeature",
    "PriceFeatureTypedDict": ".pricefeature",
    "PriceFeatureInput": ".pricefeatureinput",
    "PriceFeatureInputTypedDict": ".pricefeatureinput",
    "Function": ".priceproperties_union",
    "Parameters": ".priceproperties_union",
    "ParametersTypedDict": ".priceproperties_union",
    "PriceProperties1": ".priceproperties_union",
    "PriceProperties1TypedDict": ".priceproperties_union",
    "PriceProperties2": ".priceproperties_union",
    "PriceProperties2TypedDict": ".priceproperties_union",
    "PriceProperties3": ".priceproperties_union",
    "PriceProperties3TypedDict": ".priceproperties_union",
    "PriceProperties4": ".priceproperties_union",
    "PriceProperties4TypedDict": ".priceproperties_union",
    "PricePropertiesUnion": ".priceproperties_union",
    "PricePropertiesUnionTypedDict": ".priceproperties_union",
    "Product": ".product",
    "ProductObject": ".product",
    "ProductTypedDict": ".product",
    "PurchaseEntitlementGrantRequest": ".purchaseentitlementgrantop",
    "PurchaseEntitlementGrantRequestTypedDict": ".purchaseentitlementgrantop",
    "PurchaseGrantRequest": ".purchasegrantrequest",
    "PurchaseGrantRequestTypedDict": ".purchasegrantrequest",
    "PaymentSession": ".purchasegrantresponse",
    "PaymentSessionTypedDict": ".purchasegrantresponse",
    "PurchaseGrantResponse": ".purchasegrantresponse",
    "PurchaseGrantResponseObject": ".purchasegrantresponse",
    "PurchaseGrantResponseTypedDict": ".purchasegrantresponse",
    "RefundUsageEventRequest": ".refundusageeventop",
    "RefundUsageEventRequestBody": ".refundusageeventop",
    "RefundUsageEventRequestBodyTypedDict": ".refundusageeventop",
    "RefundUsageEventRequestTypedDict": ".refundusageeventop",
    "RejectSourceEventRequest": ".rejectsourceeventop",
    "RejectSourceEventRequestTypedDict": ".rejectsourceeventop",
    "RevenueSummaryResponse": ".revenuesummaryresponse",
    "RevenueSummaryResponseTypedDict": ".revenuesummaryresponse",
    "RevenueTrendBucket": ".revenuetrendbucket",
    "RevenueTrendBucketTypedDict": ".revenuetrendbucket",
    "From": ".rulecondition",
    "FromTypedDict": ".rulecondition",
    "Operator": ".rulecondition",
    "RuleCondition": ".rulecondition",
    "RuleConditionType": ".rulecondition",
    "RuleConditionTypedDict": ".rulecondition",
    "To": ".rulecondition",
    "ToTypedDict": ".rulecondition",
    "Value": ".rulecondition",
    "ValueTypedDict": ".rulecondition",
    "ValueUnion": ".rulecondition",
    "ValueUnionTypedDict": ".rulecondition",
    "SchemasEntitlement": ".schemas_entitlement",
    "SchemasEntitlementObject": ".schemas_entitlement",
    "SchemasEntitlementStatus": ".schemas_entitlement",
    "SchemasEntitlementTypedDict": ".schemas_entitlement",
    "SchemasPrice": ".schemas_price",
    "SchemasPriceModel": ".schemas_price",
    "SchemasPriceObject": ".schemas_price",
    "SchemasPricePaymentTerm": ".schemas_price",
    "SchemasPriceTypedDict": ".schemas_price",
    "Security": ".security",
    "SecurityTypedDict": ".security",
    "ProcessingMode": ".source",
    "Source": ".source",
    "SourceObject": ".source",
    "SourceType": ".source",
    "SourceTypedDict": ".source",
    "SourceEvent": ".sourceevent",
    "SourceEventStatus": ".sourceevent",
    "SourceEventTypedDict": ".sourceevent",
    "SourceRule": ".sourcerule",
    "SourceRuleTypedDict": ".sourcerule",
    "StaticEntitlementDetail": ".staticentitlementdetail",
    "StaticEntitlementDetailObject": ".staticentitlementdetail",
    "StaticEntitlementDetailStatus": ".staticentitlementdetail",
    "StaticEntitlementDetailTypedDict": ".staticentitlementdetail",
    "Breakdown": ".subscription",
    "BreakdownTypedDict": ".subscription",
    "PaymentPaid": ".subscription",
    "PaymentPaidTypedDict": ".subscription",
    "PaymentPending": ".subscription",
    "PaymentPendingTypedDict": ".subscription",
    "PaymentUnion": ".subscription",
    "PaymentUnionTypedDict": ".subscription",
    "Subscription": ".subscription",
    "SubscriptionObject": ".subscription",
    "SubscriptionStatusEnum": ".subscription",
    "SubscriptionTypedDict": ".subscription",
    "UnknownPaymentUnion": ".subscription",
    "SubscriptionPortal": ".subscriptionportal",
    "SubscriptionPortalObject": ".subscriptionportal",
    "SubscriptionPortalTypedDict": ".subscriptionportal",
    "TerminateSubscriptionRequest": ".terminatesubscriptionop",
    "TerminateSubscriptionRequestBody": ".terminatesubscriptionop",
    "TerminateSubscriptionRequestBodyTypedDict": ".terminatesubscriptionop",
    "TerminateSubscriptionRequestTypedDict": ".terminatesubscriptionop",
    "TestClock": ".testclock",
    "TestClockTypedDict": ".testclock",
    "UpdateBillableMetricRequest": ".updatebillablemetricop",
    "UpdateBillableMetricRequestBody": ".updatebillablemetricop",
    "UpdateBillableMetricRequestBodyTypedDict": ".updatebillablemetricop",
    "UpdateBillableMetricRequestTypedDict": ".updatebillablemetricop",
    "TaxRates": ".updatecustomerop",
    "TaxRatesTypedDict": ".updatecustomerop",
    "UpdateCustomerRequest": ".updatecustomerop",
    "UpdateCustomerRequestBody": ".updatecustomerop",
    "UpdateCustomerRequestBodyTypedDict": ".updatecustomerop",
    "UpdateCustomerRequestTypedDict": ".updatecustomerop",
    "UpdateFeatureRequest": ".updatefeatureop",
    "UpdateFeatureRequestBody": ".updatefeatureop",
    "UpdateFeatureRequestBodyTypedDict": ".updatefeatureop",
    "UpdateFeatureRequestTypedDict": ".updatefeatureop",
    "UpdateFeatureType": ".updatefeatureop",
    "UpdateFeeRequest": ".updatefeeop",
    "UpdateFeeRequestBody": ".updatefeeop",
    "UpdateFeeRequestBodyTypedDict": ".updatefeeop",
    "UpdateFeeRequestTypedDict": ".updatefeeop",
    "UpdatePlanRequest": ".updateplanop",
    "UpdatePlanRequestBody": ".updateplanop",
    "UpdatePlanRequestBodyTypedDict": ".updateplanop",
    "UpdatePlanRequestTypedDict": ".updateplanop",
    "UpdatePlanTaxBehavior": ".updateplanop",
    "UpdatePriceModel": ".updatepriceop",
    "UpdatePricePaymentTerm": ".updatepriceop",
    "UpdatePriceRequest": ".updatepriceop",
    "UpdatePriceRequestBody": ".updatepriceop",
    "UpdatePriceRequestBodyTypedDict": ".updatepriceop",
    "UpdatePriceRequestTypedDict": ".updatepriceop",
    "UpdateProductRequest": ".updateproductop",
    "UpdateProductRequestBody": ".updateproductop",
    "UpdateProductRequestBodyTypedDict": ".updateproductop",
    "UpdateProductRequestTypedDict": ".updateproductop",
    "UpdateRuleRequest": ".updaterulerequest",
    "UpdateRuleRequestTypedDict": ".updaterulerequest",
    "UpdateSourceProcessingMode": ".updatesourceop",
    "UpdateSourceRequest": ".updatesourceop",
    "UpdateSourceRequestBody": ".updatesourceop",
    "UpdateSourceRequestBodyTypedDict": ".updatesourceop",
    "UpdateSourceRequestTypedDict": ".updatesourceop",
    "UpdateSourceRuleRequest": ".updatesourceruleop",
    "UpdateSourceRuleRequestTypedDict": ".updatesourceruleop",
    "UpdateUserRequest": ".updateuserop",
    "UpdateUserRequestBody": ".updateuserop",
    "UpdateUserRequestBodyTypedDict": ".updateuserop",
    "UpdateUserRequestTypedDict": ".updateuserop",
    "UpdateUserType": ".updateuserop",
    "Billing": ".usageevent",
    "BillingTypedDict": ".usageevent",
    "Refund": ".usageevent",
    "RefundTypedDict": ".usageevent",
    "UsageEvent": ".usageevent",
    "UsageEventObject": ".usageevent",
    "UsageEventProperty": ".usageevent",
    "UsageEventPropertyTypedDict": ".usageevent",
    "UsageEventTypedDict": ".usageevent",
    "GroupedValue": ".usageresponse",
    "GroupedValueTypedDict": ".usageresponse",
    "UsageResponse": ".usageresponse",
    "UsageResponseObject": ".usageresponse",
    "UsageResponseTypedDict": ".usageresponse",
    "WindowedValue": ".usageresponse",
    "WindowedValueTypedDict": ".usageresponse",
    "State": ".user",
    "User": ".user",
    "UserObject": ".user",
    "UserOrganization": ".user",
    "UserOrganizationTypedDict": ".user",
    "UserType": ".user",
    "UserTypedDict": ".user",
    "Error": ".validationerror",
    "ErrorEnum": ".validationerror",
    "ErrorTypedDict": ".validationerror",
    "ValidTaxAddress": ".validtaxaddress",
    "ValidTaxAddressTypedDict": ".validtaxaddress",
    "VoidEntitlementGrantRequest": ".voidentitlementgrantop",
    "VoidEntitlementGrantRequestTypedDict": ".voidentitlementgrantop",
}


def __getattr__(attr_name: str) -> Any:
    return lazy_getattr(
        attr_name, package=__package__, dynamic_imports=_dynamic_imports
    )


def __dir__():
    return lazy_dir(dynamic_imports=_dynamic_imports)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .groupinvoicesummary import GroupInvoiceSummary, GroupInvoiceSummaryTypedDict
from .invoicesummary import InvoiceSummary, InvoiceSummaryTypedDict
from .paymentsummary import PaymentSummary, PaymentSummaryTypedDict
from .revenuetrendbucket import RevenueTrendBucket, RevenueTrendBucketTypedDict
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from paygentic_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class RevenueSummaryResponseTypedDict(TypedDict):
    net_revenue: str
    r"""Net collected revenue in dollars (paid invoices + completed payments)"""
    invoices: InvoiceSummaryTypedDict
    payments: PaymentSummaryTypedDict
    trend: List[RevenueTrendBucketTypedDict]
    r"""Time-bucketed revenue trend data"""
    object: Literal["revenue_summary"]
    r"""Object type identifier"""
    group_breakdown: NotRequired[List[GroupInvoiceSummaryTypedDict]]
    r"""Invoice breakdown by group dimension (only present when groupBy is specified)"""


class RevenueSummaryResponse(BaseModel):
    net_revenue: Annotated[str, pydantic.Field(alias="netRevenue")]
    r"""Net collected revenue in dollars (paid invoices + completed payments)"""

    invoices: InvoiceSummary

    payments: PaymentSummary

    trend: List[RevenueTrendBucket]
    r"""Time-bucketed revenue trend data"""

    object: Annotated[
        Annotated[
            Literal["revenue_summary"],
            AfterValidator(validate_const("revenue_summary")),
        ],
        pydantic.Field(alias="object"),
    ] = "revenue_summary"
    r"""Object type identifier"""

    group_breakdown: Annotated[
        Optional[List[GroupInvoiceSummary]], pydantic.Field(alias="groupBreakdown")
    ] = None
    r"""Invoice breakdown by group dimension (only present when groupBy is specified)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["groupBreakdown"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    RevenueSummaryResponse.model_rebuild()
except NameError:
    pass
